"""
OrderEnum Processing service module for the Basalam SDK.

This module provides access to Basalam's order processing service APIs.
"""

from .client import OrderProcessingService
from .models import (
    ResourceStats,
    ParcelStatus,
    FileResponse,
    AmountDrivers,
    City,
    User,
    Recipient,
    Customer,
    Status,
    ShippingMethodOption,
    ShippingMethod,
    Vendor,
    Product,
    Property,
    PropertyValue,
    VariationProperty,
    Variation,
    ParcelItem,
    Parcel,
    Order,
    ParcelResponse,
    ParcelSummaryResponse,
    ParcelsResponse,
    OrdersResponse,
    OrderStatsResponse,
    OrderParcelFilter,
    OrderFilter,
    ItemFilter,
)

__all__ = [
    "OrderProcessingService",
    "ResourceStats",
    "ParcelStatus",
    "FileResponse",
    "AmountDrivers",
    "City",
    "User",
    "Recipient",
    "Customer",
    "Status",
    "ShippingMethodOption",
    "ShippingMethod",
    "Vendor",
    "Product",
    "Property",
    "PropertyValue",
    "VariationProperty",
    "Variation",
    "ParcelItem",
    "Parcel",
    "Order",
    "ParcelResponse",
    "ParcelSummaryResponse",
    "ParcelsResponse",
    "OrdersResponse",
    "OrderStatsResponse",
    "OrderParcelFilter",
    "OrderFilter",
    "ItemFilter",
]



"""
Models for the Webhook service API.
"""
from datetime import datetime
from enum import Enum
from typing import List, Optional, Dict, Any

from pydantic import BaseModel


class RequestMethodType(str, Enum):
    POST = "POST"
    PUT = "PUT"
    PATCH = "PATCH"


class ServiceResource(BaseModel):
    """Service resource model."""
    id: Optional[int] = None
    title: Optional[str] = None
    description: Optional[str] = None
    is_verified: Optional[bool] = None
    is_active: Optional[bool] = None
    created_at: Optional[str] = None


class ServiceListResource(BaseModel):
    """Service list resource model."""
    data: Optional[List[ServiceResource]] = None
    result_count: Optional[int] = None
    total_count: Optional[int] = None
    total_page: Optional[int] = None
    page: Optional[int] = None
    per_page: Optional[int] = None


class CreateServiceRequest(BaseModel):
    """Create service request model."""
    title: str
    description: str


class WebhookResource(BaseModel):
    """Webhook resource model."""
    id: int
    service_id: int
    events: Optional[List[Dict[str, Any]]] = None
    request_headers: Optional[Dict[str, Any]] = None
    request_method: Optional[RequestMethodType] = None
    url: Optional[str] = None
    is_active: Optional[bool] = None
    created_at: Optional[datetime] = None
    updated_at: Optional[datetime] = None


class WebhookListResource(BaseModel):
    """Webhook list resource model."""
    data: Optional[List[WebhookResource]] = None
    result_count: Optional[int] = None
    total_count: Optional[int] = None
    total_page: Optional[int] = None
    page: Optional[int] = None
    per_page: Optional[int] = None


class CreateWebhookRequest(BaseModel):
    """Create webhook request model."""
    service_id: Optional[int] = None
    event_ids: List[int]
    request_headers: Optional[str] = None
    request_method: RequestMethodType
    url: str
    is_active: Optional[bool] = None
    register_me: Optional[bool] = None


class EventResource(BaseModel):
    """Event resource model."""
    id: int
    name: str
    description: Optional[str] = None
    sample_data: Optional[Dict[str, Any]] = None
    scopes: Optional[str] = None


class EventListResource(BaseModel):
    """Event list resource model."""
    data: Optional[List[EventResource]] = None
    result_count: Optional[int] = None
    total_count: Optional[int] = None
    total_page: Optional[int] = None
    page: Optional[int] = None
    per_page: Optional[int] = None


class UpdateWebhookRequest(BaseModel):
    """Request model for updating a webhook."""
    event_ids: Optional[List[int]] = None
    request_headers: Optional[str] = None
    request_method: Optional[RequestMethodType] = None
    url: Optional[str] = None
    is_active: Optional[bool] = None


class DeleteWebhookResponse(BaseModel):
    """Response model for webhook deletion."""
    id: int
    deleted_at: Optional[datetime] = None


class WebhookLogResource(BaseModel):
    """Response model for webhook log resources."""
    id: int
    user_id: int
    status_code: int
    request: Dict[str, Any]
    response: str
    created_at: Optional[datetime] = None


class WebhookLogListResource(BaseModel):
    """Response model for list of webhook logs."""
    data: Optional[List[WebhookLogResource]] = None
    result_count: Optional[int] = None
    total_count: Optional[int] = None
    total_page: Optional[int] = None
    page: Optional[int] = None
    per_page: Optional[int] = None


class RegisterClientRequest(BaseModel):
    """Request model for registering a client to a webhook."""
    webhook_id: int


class UnRegisterClientRequest(BaseModel):
    """Request model for unregistering a client from a webhook."""
    webhook_id: int
    customer_id: Optional[int] = None


class UnRegisterClientResponse(BaseModel):
    """Response model for client unregistration."""
    webhook_id: int
    customer_id: int
    deleted_at: Optional[datetime] = None


class ClientResource(BaseModel):
    """Response model for client resources."""
    id: int
    customer_id: int
    webhook_id: int
    created_at: Optional[datetime] = None
    updated_at: Optional[datetime] = None


class ClientListResource(BaseModel):
    """Response model for list of clients."""
    data: Optional[List[ClientResource]] = None
    result_count: Optional[int] = None
    total_count: Optional[int] = None
    total_page: Optional[int] = None
    page: Optional[int] = None
    per_page: Optional[int] = None


class WebhookRegisteredOnResource(BaseModel):
    """Response model for webhook registration resources."""
    id: int
    service_id: int
    customer_id: int
    events: List[Dict[str, Any]]
    is_active: Optional[bool] = None
    registered_at: Optional[datetime] = None


class WebhookRegisteredOnListResource(BaseModel):
    """Response model for list of webhook registrations."""
    data: Optional[List[WebhookRegisteredOnResource]] = None
    result_count: Optional[int] = None
    total_count: Optional[int] = None
    total_page: Optional[int] = None
    page: Optional[int] = None
    per_page: Optional[int] = None



"""Type stubs for BasalamClient - provides IDE autocomplete support."""
from typing import Any, Dict, List, Optional, Union, BinaryIO
from .auth import BaseAuth, Scope
from .config import BasalamConfig

class BasalamClient:
    """Main client for interacting with the Basalam API."""

    def __init__(self, auth: BaseAuth, config: Optional[BasalamConfig] = None) -> None: ...

    # Service attributes
    core: Any
    chat: Any
    order: Any
    order_processing: Any
    search: Any
    upload: Any
    wallet: Any
    webhook: Any

    # Context manager and auth methods
    async def __aenter__(self) -> BasalamClient: ...
    async def __aexit__(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> None: ...
    def has_scope(self, scope: Union[str, Scope]) -> bool: ...
    def get_granted_scopes(self) -> List[str]: ...
    async def refresh_auth_token(self) -> None: ...
    def refresh_auth_token_sync(self) -> None: ...

    # Service methods (dynamically delegated)
    async def create_discount( self, vendor_id: int, request: CreateDiscountRequestSchema ) -> Dict[str, Any]: ...
    def create_discount_sync( self, vendor_id: int, request: CreateDiscountRequestSchema ) -> Dict[str, Any]: ...
    async def create_product( self, vendor_id: int, request: ProductRequestSchema, photo_files: Optional[List[BinaryIO]] = None, video_file: Optional[BinaryIO] = None ) -> ProductResponseSchema: ...
    def create_product_sync( self, vendor_id: int, request: ProductRequestSchema, photo_files: Optional[List[BinaryIO]] = None, video_file: Optional[BinaryIO] = None ) -> ProductResponseSchema: ...
    async def create_products_bulk_action_request( self, vendor_id: int, request: BulkProductsUpdateRequestSchema ) -> BulkProductsUpdateResponseSchema: ...
    def create_products_bulk_action_request_sync( self, vendor_id: int, request: BulkProductsUpdateRequestSchema ) -> BulkProductsUpdateResponseSchema: ...
    async def create_shelve( self, request: ShelveSchema ) -> Dict[str, Any]: ...
    def create_shelve_sync( self, request: ShelveSchema ) -> Dict[str, Any]: ...
    async def create_user_bank_account( self, user_id: int, request: UserCardsSchema, prefer: Optional[str] = "return=minimal" ) -> Dict[str, Any]: ...
    def create_user_bank_account_sync( self, user_id: int, request: UserCardsSchema, prefer: Optional[str] = "return=minimal" ) -> Dict[str, Any]: ...
    async def create_user_mobile_change_request( self, user_id: int, request: ChangeUserMobileRequestSchema ) -> ResultResponse: ...
    def create_user_mobile_change_request_sync( self, user_id: int, request: ChangeUserMobileRequestSchema ) -> ResultResponse: ...
    async def create_user_mobile_confirmation_request( self, user_id: int ) -> ResultResponse: ...
    def create_user_mobile_confirmation_request_sync( self, user_id: int ) -> ResultResponse: ...
    async def create_vendor( self, user_id: int, request: CreateVendorSchema ) -> PublicVendorResponse: ...
    async def create_vendor_mobile_change_confirmation( self, vendor_id: int, request: ChangeVendorMobileConfirmSchema ) -> ResultResponse: ...
    def create_vendor_mobile_change_confirmation_sync( self, vendor_id: int, request: ChangeVendorMobileConfirmSchema ) -> ResultResponse: ...
    async def create_vendor_mobile_change_request( self, vendor_id: int, request: ChangeVendorMobileRequestSchema ) -> ResultResponse: ...
    def create_vendor_mobile_change_request_sync( self, vendor_id: int, request: ChangeVendorMobileRequestSchema ) -> ResultResponse: ...
    def create_vendor_sync( self, user_id: int, request: CreateVendorSchema ) -> PublicVendorResponse: ...
    async def delete_discount( self, vendor_id: int, request: DeleteDiscountRequestSchema ) -> Dict[str, Any]: ...
    def delete_discount_sync( self, vendor_id: int, request: DeleteDiscountRequestSchema ) -> Dict[str, Any]: ...
    async def delete_shelve( self, shelve_id: int ) -> Dict[str, Any]: ...
    async def delete_shelve_product( self, shelve_id: int, product_id: int ) -> Dict[str, Any]: ...
    def delete_shelve_product_sync( self, shelve_id: int, product_id: int ) -> Dict[str, Any]: ...
    def delete_shelve_sync( self, shelve_id: int ) -> Dict[str, Any]: ...
    async def delete_user_bank_account( self, user_id: int, bank_account_id: int ) -> Dict[str, Any]: ...
    def delete_user_bank_account_sync( self, user_id: int, bank_account_id: int ) -> Dict[str, Any]: ...
    async def get_categories(self) -> CategoriesResponse: ...
    def get_categories_sync(self) -> CategoriesResponse: ...
    async def get_category(self, category_id: int) -> CategoryResponse: ...
    async def get_category_attributes( self, category_id: int, product_id: Optional[int] = None, vendor_id: Optional[int] = None, exclude_multi_selects: bool = True ) -> AttributesResponse: ...
    def get_category_attributes_sync( self, category_id: int, product_id: Optional[int] = None, vendor_id: Optional[int] = None, exclude_multi_selects: bool = True ) -> AttributesResponse: ...
    def get_category_sync(self, category_id: int) -> CategoryResponse: ...
    async def get_current_user(self) -> PrivateUserResponse: ...
    def get_current_user_sync(self) -> PrivateUserResponse: ...
    async def get_default_shipping_methods(self) -> List[ShippingMethodResponse]: ...
    def get_default_shipping_methods_sync(self) -> List[ShippingMethodResponse]: ...
    async def get_product( self, product_id: int, prefer: Optional[str] = "return=minimal" ) -> ProductResponseSchema: ...
    async def get_product_shelves( self, product_id: int ) -> List[ProductShelfResponse]: ...
    def get_product_shelves_sync( self, product_id: int ) -> List[ProductShelfResponse]: ...
    def get_product_sync( self, product_id: int, prefer: Optional[str] = "return=minimal" ) -> ProductResponseSchema: ...
    async def get_products( self, query_params: Optional[GetProductsQuerySchema] = None, prefer: Optional[str] = "return=minimal" ) -> ProductListResponse: ...
    async def get_products_bulk_action_requests( self, vendor_id: int, page: int = 1, per_page: int = 30 ) -> BulkProductsUpdatesListResponse: ...
    async def get_products_bulk_action_requests_count( self, vendor_id: int ) -> BulkProductsUpdatesCountResponse: ...
    def get_products_bulk_action_requests_count_sync( self, vendor_id: int ) -> BulkProductsUpdatesCountResponse: ...
    def get_products_bulk_action_requests_sync( self, vendor_id: int, page: int = 1, per_page: int = 30 ) -> BulkProductsUpdatesListResponse: ...
    def get_products_sync( self, query_params: Optional[GetProductsQuerySchema] = None, prefer: Optional[str] = "return=minimal" ) -> ProductListResponse: ...
    async def get_products_unsuccessful_bulk_action_requests( self, request_id: int, page: int = 1, per_page: int = 20 ) -> UnsuccessfulBulkUpdateProducts: ...
    def get_products_unsuccessful_bulk_action_requests_sync( self, request_id: int, page: int = 1, per_page: int = 20 ) -> UnsuccessfulBulkUpdateProducts: ...
    async def get_shelve_products( self, shelve_id: int, title: Optional[str] = None ) -> Dict[str, Any]: ...
    def get_shelve_products_sync( self, shelve_id: int, title: Optional[str] = None ) -> Dict[str, Any]: ...
    async def get_shipping_methods( self, ids: Optional[List[int]] = None, vendor_ids: Optional[List[int]] = None, include_deleted: Optional[bool] = None, page: int = 1, per_page: int = 10 ) -> ShippingMethodListResponse: ...
    def get_shipping_methods_sync( self, ids: Optional[List[int]] = None, vendor_ids: Optional[List[int]] = None, include_deleted: Optional[bool] = None, page: int = 1, per_page: int = 10 ) -> ShippingMethodListResponse: ...
    async def get_user_bank_accounts( self, user_id: int, prefer: Optional[str] = "return=minimal" ) -> List[Dict[str, Any]]: ...
    def get_user_bank_accounts_sync( self, user_id: int, prefer: Optional[str] = "return=minimal" ) -> List[Dict[str, Any]]: ...
    async def get_vendor( self, vendor_id: int, prefer: Optional[str] = "return=minimal" ) -> Union[PublicVendorResponse, PrivateVendorResponse]: ...
    async def get_vendor_products( self, vendor_id: int, query_params: Optional[GetVendorProductsSchema] = None ) -> ProductListResponse: ...
    def get_vendor_products_sync( self, vendor_id: int, query_params: Optional[GetVendorProductsSchema] = None ) -> ProductListResponse: ...
    def get_vendor_sync( self, vendor_id: int, prefer: Optional[str] = "return=minimal" ) -> Union[PublicVendorResponse, PrivateVendorResponse]: ...
    async def get_working_shipping_methods( self, vendor_id: int ) -> List[ShippingMethodResponse]: ...
    def get_working_shipping_methods_sync( self, vendor_id: int ) -> List[ShippingMethodResponse]: ...
    async def update_bulk_products( self, vendor_id: int, request: BatchUpdateProductsRequest ) -> List[UpdateProductResponseItem]: ...
    def update_bulk_products_sync( self, vendor_id: int, request: BatchUpdateProductsRequest ) -> List[UpdateProductResponseItem]: ...
    async def update_product( self, product_id: int, request: ProductRequestSchema, photo_files: Optional[List[BinaryIO]] = None, video_file: Optional[BinaryIO] = None ) -> ProductResponseSchema: ...
    def update_product_sync( self, product_id: int, request: ProductRequestSchema, photo_files: Optional[List[BinaryIO]] = None, video_file: Optional[BinaryIO] = None ) -> ProductResponseSchema: ...
    async def update_product_variation( self, product_id: int, variation_id: int, request: UpdateProductVariationSchema ) -> ProductResponseSchema: ...
    def update_product_variation_sync( self, product_id: int, variation_id: int, request: UpdateProductVariationSchema ) -> ProductResponseSchema: ...
    async def update_shelve( self, shelve_id: int, request: ShelveSchema ) -> Dict[str, Any]: ...
    async def update_shelve_products( self, shelve_id: int, request: UpdateShelveProductsSchema ) -> Dict[str, Any]: ...
    def update_shelve_products_sync( self, shelve_id: int, request: UpdateShelveProductsSchema ) -> Dict[str, Any]: ...
    def update_shelve_sync( self, shelve_id: int, request: ShelveSchema ) -> Dict[str, Any]: ...
    async def update_shipping_methods( self, vendor_id: int, request: UpdateShippingMethodSchema ) -> List[ShippingMethodResponse]: ...
    def update_shipping_methods_sync( self, vendor_id: int, request: UpdateShippingMethodSchema ) -> List[ShippingMethodResponse]: ...
    async def update_user_bank_account( self, bank_account_id: int, request: UpdateUserBankInformationSchema ) -> Dict[str, Any]: ...
    def update_user_bank_account_sync( self, bank_account_id: int, request: UpdateUserBankInformationSchema ) -> Dict[str, Any]: ...
    async def update_user_verification( self, user_id: int, request: UserVerificationSchema ) -> PrivateUserResponse: ...
    def update_user_verification_sync( self, user_id: int, request: UserVerificationSchema ) -> PrivateUserResponse: ...
    async def update_vendor( self, vendor_id: int, request: UpdateVendorSchema ) -> PublicVendorResponse: ...
    async def update_vendor_status( self, vendor_id: int, request: UpdateVendorStatusSchema ) -> UpdateVendorStatusResponse: ...
    def update_vendor_status_sync( self, vendor_id: int, request: UpdateVendorStatusSchema ) -> UpdateVendorStatusResponse: ...
    def update_vendor_sync( self, vendor_id: int, request: UpdateVendorSchema ) -> PublicVendorResponse: ...
    async def verify_user_bank_account( self, user_id: int, request: UserVerifyBankInformationSchema ) -> Dict[str, Any]: ...
    async def verify_user_bank_account_otp( self, user_id: int, request: UserCardsOtpSchema ) -> Dict[str, Any]: ...
    def verify_user_bank_account_otp_sync( self, user_id: int, request: UserCardsOtpSchema ) -> Dict[str, Any]: ...
    def verify_user_bank_account_sync( self, user_id: int, request: UserVerifyBankInformationSchema ) -> Dict[str, Any]: ...
    async def verify_user_mobile_change_request( self, user_id: int, request: ChangeUserMobileConfirmSchema ) -> ResultResponse: ...
    def verify_user_mobile_change_request_sync( self, user_id: int, request: ChangeUserMobileConfirmSchema ) -> ResultResponse: ...
    async def verify_user_mobile_confirmation_request( self, user_id: int, request: ConfirmCurrentUserMobileConfirmSchema ) -> ResultResponse: ...
    def verify_user_mobile_confirmation_request_sync( self, user_id: int, request: ConfirmCurrentUserMobileConfirmSchema ) -> ResultResponse: ...
    async def create_chat( self, request: CreateChatRequest, x_creation_tags: Optional[str] = None ) -> CreateChatResponse: ...
    def create_chat_sync( self, request: CreateChatRequest, x_creation_tags: Optional[str] = None ) -> CreateChatResponse: ...
    async def create_message( self, request: MessageRequest, user_agent: Optional[str] = None ) -> MessageResponse: ...
    def create_message_sync( self, request: MessageRequest, user_agent: Optional[str] = None ) -> MessageResponse: ...
    async def delete_chats( self, request: DeleteChatsRequest ) -> BooleanResponse: ...
    def delete_chats_sync( self, request: DeleteChatsRequest ) -> BooleanResponse: ...
    async def delete_message( self, request: DeleteMessageRequest ) -> BooleanResponse: ...
    def delete_message_sync( self, request: DeleteMessageRequest ) -> BooleanResponse: ...
    async def delete_webhook(self, token: str) -> BotApiResponse: ...
    async def delete_webhook_delete(self, token: str) -> BotApiResponse: ...
    def delete_webhook_delete_sync(self, token: str) -> BotApiResponse: ...
    async def delete_webhook_post(self, token: str) -> BotApiResponse: ...
    def delete_webhook_post_sync(self, token: str) -> BotApiResponse: ...
    def delete_webhook_sync(self, token: str) -> BotApiResponse: ...
    async def edit_message( self, request: EditMessageRequest, x_client_info: Optional[str] = None ) -> EditMessageResponse: ...
    def edit_message_sync( self, request: EditMessageRequest, x_client_info: Optional[str] = None ) -> EditMessageResponse: ...
    async def forward_message( self, request: ForwardMessageRequest, user_agent: Optional[str] = None ) -> BooleanResponse: ...
    def forward_message_sync( self, request: ForwardMessageRequest, user_agent: Optional[str] = None ) -> BooleanResponse: ...
    async def get_chats( self, request: GetChatsRequest ) -> ChatListResponse: ...
    def get_chats_sync( self, request: GetChatsRequest ) -> ChatListResponse: ...
    async def get_me(self, token: str) -> BotApiResponse: ...
    async def get_me_post(self, token: str) -> BotApiResponse: ...
    def get_me_post_sync(self, token: str) -> BotApiResponse: ...
    def get_me_sync(self, token: str) -> BotApiResponse: ...
    async def get_messages( self, request: Optional[GetMessagesRequest] = None ) -> GetMessagesResponse: ...
    def get_messages_sync( self, request: Optional[GetMessagesRequest] = None ) -> GetMessagesResponse: ...
    async def get_unseen_chat_count(self) -> UnseenChatCountResponse: ...
    def get_unseen_chat_count_sync(self) -> UnseenChatCountResponse: ...
    async def get_webhook_info(self, token: str) -> BotApiResponse: ...
    async def get_webhook_info_post(self, token: str) -> BotApiResponse: ...
    def get_webhook_info_post_sync(self, token: str) -> BotApiResponse: ...
    def get_webhook_info_sync(self, token: str) -> BotApiResponse: ...
    async def log_out(self, token: str) -> BotApiResponse: ...
    async def log_out_post(self, token: str) -> BotApiResponse: ...
    def log_out_post_sync(self, token: str) -> BotApiResponse: ...
    def log_out_sync(self, token: str) -> BotApiResponse: ...
    async def create_invoice_payment( self, invoice_id: int, request: CreatePaymentRequestModel ) -> Dict[str, Any]: ...
    def create_invoice_payment_sync( self, invoice_id: int, request: CreatePaymentRequestModel ) -> Dict[str, Any]: ...
    async def create_payment_callback( self, payment_id: int, request: PaymentVerifyRequestModel ) -> Dict[str, Any]: ...
    def create_payment_callback_sync( self, payment_id: int, request: PaymentVerifyRequestModel ) -> Dict[str, Any]: ...
    async def get_baskets(self, refresh: bool = False) -> BasketResponse: ...
    def get_baskets_sync(self, refresh: bool = False) -> BasketResponse: ...
    async def get_payable_invoices( self, page: int, per_page: int ) -> Dict[str, Any]: ...
    def get_payable_invoices_sync( self, page: int, per_page: int ) -> Dict[str, Any]: ...
    async def get_payment_callback( self, payment_id: int, request: PaymentCallbackRequestModel ) -> Dict[str, Any]: ...
    def get_payment_callback_sync( self, payment_id: int, request: PaymentCallbackRequestModel ) -> Dict[str, Any]: ...
    async def get_product_variation_status(self, product_id: int) -> Dict[str, Any]: ...
    def get_product_variation_status_sync(self, product_id: int) -> Dict[str, Any]: ...
    async def get_unpaid_invoices( self, invoice_id: Optional[int] = None, status: Optional[UnpaidInvoiceStatusEnum] = None, page: int = 1, per_page: int = 20, sort: OrderEnum = OrderEnum.DESC ) -> Dict[str, Any]: ...
    def get_unpaid_invoices_sync( self, invoice_id: Optional[int] = None, status: Optional[UnpaidInvoiceStatusEnum] = None, page: int = 1, per_page: int = 20, sort: OrderEnum = OrderEnum.DESC ) -> Dict[str, Any]: ...
    async def get_customer_order(self, order_id: int) -> Order: ...
    async def get_customer_order_item(self, item_id: int) -> CustomerItemResponse: ...
    def get_customer_order_item_sync(self, item_id: int) -> CustomerItemResponse: ...
    async def get_customer_order_items( self, filters: Optional[ItemFilter] = None ) -> CustomerItemsResponse: ...
    def get_customer_order_items_sync( self, filters: Optional[ItemFilter] = None ) -> CustomerItemsResponse: ...
    async def get_customer_order_parcel_hints(self, order_id: int) -> ParcelHintsResponse: ...
    def get_customer_order_parcel_hints_sync(self, order_id: int) -> ParcelHintsResponse: ...
    def get_customer_order_sync(self, order_id: int) -> Order: ...
    async def get_customer_orders( self, filters: Optional[OrderFilter] = None ) -> OrdersResponse: ...
    def get_customer_orders_sync( self, filters: Optional[OrderFilter] = None ) -> OrdersResponse: ...
    async def get_order_parcel(self, parcel_id: int) -> ParcelResponse: ...
    def get_order_parcel_sync(self, parcel_id: int) -> ParcelResponse: ...
    async def get_orders_stats( self, resource_count: ResourceStats, vendor_id: Optional[int] = None, product_id: Optional[int] = None, customer_id: Optional[int] = None, coupon_code: Optional[str] = None, cache_control: Optional[str] = None ) -> OrderStatsResponse: ...
    def get_orders_stats_sync( self, resource_count: ResourceStats, vendor_id: Optional[int] = None, product_id: Optional[int] = None, customer_id: Optional[int] = None, coupon_code: Optional[str] = None, cache_control: Optional[str] = None ) -> OrderStatsResponse: ...
    async def get_vendor_orders_parcels( self, filters: Optional[OrderParcelFilter] = None ) -> ParcelsResponse: ...
    def get_vendor_orders_parcels_sync( self, filters: Optional[OrderParcelFilter] = None ) -> ParcelsResponse: ...
    async def search_products(self, request: ProductSearchModel) -> Dict[str, Any]: ...
    def search_products_sync(self, request: ProductSearchModel) -> Dict[str, Any]: ...
    async def upload_file( self, file: BinaryIO, file_type: UserUploadFileTypeEnum, custom_unique_name: Optional[str] = None, expire_minutes: Optional[int] = None ) -> FileResponse: ...
    def upload_file_sync( self, file: BinaryIO, file_type: UserUploadFileTypeEnum, custom_unique_name: Optional[str] = None, expire_minutes: Optional[int] = None ) -> FileResponse: ...
    async def create_expense( self, user_id: int, request: SpendCreditRequest, x_operator_id: Optional[int] = None ) -> SpendResponse: ...
    def create_expense_sync( self, user_id: int, request: SpendCreditRequest, x_operator_id: Optional[int] = None ) -> SpendResponse: ...
    async def delete_expense( self, user_id: int, expense_id: int, rollback_reason_id: int, x_operator_id: Optional[int] = None ) -> SpendResponse: ...
    async def delete_expense_by_ref( self, user_id: int, reason_id: int, reference_id: int, rollback_reason_id: int, x_operator_id: Optional[int] = None ) -> SpendResponse: ...
    def delete_expense_by_ref_sync( self, user_id: int, reason_id: int, reference_id: int, rollback_reason_id: int, x_operator_id: Optional[int] = None ) -> SpendResponse: ...
    def delete_expense_sync( self, user_id: int, expense_id: int, rollback_reason_id: int, x_operator_id: Optional[int] = None ) -> SpendResponse: ...
    async def get_balance( self, user_id: int, filters: List[BalanceFilter] = None, x_operator_id: Optional[int] = None ) -> Dict[str, Any]: ...
    def get_balance_sync( self, user_id: int, filters: List[BalanceFilter] = None, x_operator_id: Optional[int] = None ) -> Dict[str, Any]: ...
    async def get_expense( self, user_id: int, expense_id: int, x_operator_id: Optional[int] = None ) -> SpendResponse: ...
    async def get_expense_by_ref( self, user_id: int, reason_id: int, reference_id: int, x_operator_id: Optional[int] = None ) -> SpendResponse: ...
    def get_expense_by_ref_sync( self, user_id: int, reason_id: int, reference_id: int, x_operator_id: Optional[int] = None ) -> SpendResponse: ...
    def get_expense_sync( self, user_id: int, expense_id: int, x_operator_id: Optional[int] = None ) -> SpendResponse: ...
    async def get_transactions( self, user_id: int, page: int = 1, per_page: int = 50, x_operator_id: Optional[int] = None ) -> HistoryPaginationResponse: ...
    def get_transactions_sync( self, user_id: int, page: int = 1, per_page: int = 50, x_operator_id: Optional[int] = None ) -> HistoryPaginationResponse: ...
    async def create_webhook(self, request: CreateWebhookRequest) -> WebhookResource: ...
    async def create_webhook_service(self, request: CreateServiceRequest) -> ServiceResource: ...
    def create_webhook_service_sync(self, request: CreateServiceRequest) -> ServiceResource: ...
    def create_webhook_sync(self, request: CreateWebhookRequest) -> WebhookResource: ...
    async def get_registered_webhooks( self, page: Optional[int] = 1, per_page: Optional[int] = 10, service_id: Optional[int] = None ) -> WebhookRegisteredOnListResource: ...
    def get_registered_webhooks_sync( self, page: Optional[int] = 1, per_page: Optional[int] = 10, service_id: Optional[int] = None ) -> WebhookRegisteredOnListResource: ...
    async def get_webhook_customers( self, page: Optional[int] = 1, per_page: Optional[int] = 10, webhook_id: Optional[int] = None ) -> ClientListResource: ...
    def get_webhook_customers_sync( self, page: Optional[int] = 1, per_page: Optional[int] = 10, webhook_id: Optional[int] = None ) -> ClientListResource: ...
    async def get_webhook_events(self) -> EventListResource: ...
    def get_webhook_events_sync(self) -> EventListResource: ...
    async def get_webhook_logs(self, webhook_id: int) -> WebhookLogListResource: ...
    def get_webhook_logs_sync(self, webhook_id: int) -> WebhookLogListResource: ...
    async def get_webhook_services(self) -> ServiceListResource: ...
    def get_webhook_services_sync(self) -> ServiceListResource: ...
    async def get_webhooks( self, service_id: Optional[int] = None, event_ids: Optional[str] = None ) -> WebhookListResource: ...
    def get_webhooks_sync( self, service_id: Optional[int] = None, event_ids: Optional[str] = None ) -> WebhookListResource: ...
    async def register_webhook(self, request: RegisterClientRequest) -> ClientResource: ...
    def register_webhook_sync(self, request: RegisterClientRequest) -> ClientResource: ...
    async def unregister_webhook(self, request: UnRegisterClientRequest) -> UnRegisterClientResponse: ...
    def unregister_webhook_sync(self, request: UnRegisterClientRequest) -> UnRegisterClientResponse: ...
    async def update_webhook( self, webhook_id: int, request: UpdateWebhookRequest ) -> WebhookResource: ...
    def update_webhook_sync( self, webhook_id: int, request: UpdateWebhookRequest ) -> WebhookResource: ...


"""
Client for the Basalam OrderEnum Processing Service.

This module provides a client for interacting with Basalam's order processing service.
"""

import logging
from typing import Optional

from .models import (
    OrdersResponse,
    CustomerItemResponse,
    CustomerItemsResponse,
    ParcelResponse,
    OrderStatsResponse,
    ResourceStats,
    OrderFilter,
    ItemFilter,
    OrderParcelFilter,
    ParcelsResponse, Order,
    ParcelHintsResponse,
)
from ..base_client import BaseClient

logger = logging.getLogger(__name__)


class OrderProcessingService(BaseClient):
    """
    Client for the Basalam OrderEnum Processing Service API.

    This client provides methods for interacting with customer orders,
    vendor orders, and order statistics.
    """

    def __init__(self, **kwargs):
        """
        Initialize the order processing service client.
        """
        super().__init__(service="order-processing", **kwargs)

    async def get_customer_orders(
            self,
            filters: Optional[OrderFilter] = None
    ) -> OrdersResponse:
        """
        Get a list of customer orders.

        Args:
            filters: Optional filters to apply to the query.

        Returns:
            The response containing the list of orders.
        """
        endpoint = "/v1/customer-orders"
        filters = filters or OrderFilter()
        params = filters.model_dump(exclude_none=True)

        # Handle field mapping for API compatibility
        if "items_title" in params:
            params["items.title"] = params.pop("items_title")
        if "parcel_estimate_send_at" in params:
            params["parcel.estimate_send_at"] = params.pop("parcel_estimate_send_at")
        if "parcel_statuses" in params:
            params["parcel.statuses"] = params.pop("parcel_statuses")

        response = await self._get(endpoint, params=params)
        return OrdersResponse(**response)

    def get_customer_orders_sync(
            self,
            filters: Optional[OrderFilter] = None
    ) -> OrdersResponse:
        """
        Get a list of orders (synchronous version).

        Args:
            filters: Optional filters to apply to the query.

        Returns:
            The response containing the list of orders.
        """
        endpoint = "/v1/customer-orders"
        filters = filters or OrderFilter()
        params = filters.model_dump(exclude_none=True)

        # Handle field mapping for API compatibility
        if "items_title" in params:
            params["items.title"] = params.pop("items_title")
        if "parcel_estimate_send_at" in params:
            params["parcel.estimate_send_at"] = params.pop("parcel_estimate_send_at")
        if "parcel_statuses" in params:
            params["parcel.statuses"] = params.pop("parcel_statuses")

        response = self._get_sync(endpoint, params=params)
        return OrdersResponse(**response)

    async def get_customer_order(self, order_id: int) -> Order:
        """
        Get details of a specific order.

        Args:
            order_id: The ID of the order to retrieve.

        Returns:
            The response containing the order details.
        """
        endpoint = f"/v1/customer-orders/{order_id}"
        response = await self._get(endpoint)
        return Order(**response)

    def get_customer_order_sync(self, order_id: int) -> Order:
        """
        Get details of a specific order (synchronous version).

        Args:
            order_id: The ID of the order to retrieve.

        Returns:
            The response containing the order details.
        """
        endpoint = f"/v1/customer-orders/{order_id}"
        response = self._get_sync(endpoint)
        return Order(**response)

    async def get_customer_order_parcel_hints(self, order_id: int) -> ParcelHintsResponse:
        """
        Get parcel hints for a specific customer order.

        Args:
            order_id: The ID of the order to retrieve parcel hints for.

        Returns:
            The response containing the parcel hints.
        """
        endpoint = f"/v1/customer-orders/{order_id}/parcel-hints"
        response = await self._get(endpoint)
        return ParcelHintsResponse(**response)

    def get_customer_order_parcel_hints_sync(self, order_id: int) -> ParcelHintsResponse:
        """
        Get parcel hints for a specific customer order (synchronous version).

        Args:
            order_id: The ID of the order to retrieve parcel hints for.

        Returns:
            The response containing the parcel hints.
        """
        endpoint = f"/v1/customer-orders/{order_id}/parcel-hints"
        response = self._get_sync(endpoint)
        return ParcelHintsResponse(**response)

    async def get_customer_order_items(
            self,
            filters: Optional[ItemFilter] = None
    ) -> CustomerItemsResponse:
        """
        Get a list of order items.

        Args:
            filters: Optional filters to apply to the query.

        Returns:
            The response containing the list of items.
        """
        endpoint = "/v1/customer-orders/items"
        filters = filters or ItemFilter()
        params = filters.model_dump(exclude_none=True)

        response = await self._get(endpoint, params=params)
        return CustomerItemsResponse(**response)

    def get_customer_order_items_sync(
            self,
            filters: Optional[ItemFilter] = None
    ) -> CustomerItemsResponse:
        """
        Get a list of order items (synchronous version).

        Args:
            filters: Optional filters to apply to the query.

        Returns:
            The response containing the list of items.
        """
        endpoint = "/v1/customer-orders/items"
        filters = filters or ItemFilter()
        params = filters.model_dump(exclude_none=True)

        response = self._get_sync(endpoint, params=params)
        return CustomerItemsResponse(**response)

    async def get_customer_order_item(self, item_id: int) -> CustomerItemResponse:
        """
        Get details of a specific order item.

        Args:
            item_id: The ID of the item to retrieve.

        Returns:
            The response containing the item details.
        """
        endpoint = f"/v1/customer-orders/items/{item_id}"
        response = await self._get(endpoint)
        return CustomerItemResponse(**response)

    def get_customer_order_item_sync(self, item_id: int) -> CustomerItemResponse:
        """
        Get details of a specific order item (synchronous version).

        Args:
            item_id: The ID of the item to retrieve.

        Returns:
            The response containing the item details.
        """
        endpoint = f"/v1/customer-orders/items/{item_id}"
        response = self._get_sync(endpoint)
        return CustomerItemResponse(**response)

    async def get_vendor_orders_parcels(
            self,
            filters: Optional[OrderParcelFilter] = None
    ) -> ParcelsResponse:
        """
        Get a list of orders parcels.

        Args:
            filters: Optional filters to apply to the query.

        Returns:
            The response containing the list of parcels.
        """
        endpoint = "/v1/vendor-parcels"
        filters = filters or OrderParcelFilter()

        params = {}
        if filters.created_at:
            params["created_at"] = filters.created_at
        if filters.cursor:
            params["cursor"] = filters.cursor
        if filters.estimate_send_at:
            params["estimate_send_at"] = filters.estimate_send_at
        if filters.ids:
            params["ids"] = filters.ids
        if filters.items_customer_ids:
            params["items.customer_ids"] = filters.items_customer_ids
        if filters.items_order_ids:
            params["items.order_ids"] = filters.items_order_ids
        if filters.items_product_ids:
            params["items.product_ids"] = filters.items_product_ids
        if filters.items_vendor_ids:
            params["items.vendor_ids"] = filters.items_vendor_ids
        if filters.per_page:
            params["per_page"] = filters.per_page
        if filters.sort:
            params["sort"] = filters.sort
        if filters.statuses:
            params["statuses"] = ",".join(str(status.value) for status in filters.statuses)

        response = await self._get(endpoint, params=params)
        return ParcelsResponse(**response)

    def get_vendor_orders_parcels_sync(
            self,
            filters: Optional[OrderParcelFilter] = None
    ) -> ParcelsResponse:
        """
        Get a list of orders parcels (synchronous version).

        Args:
            filters: Optional filters to apply to the query.

        Returns:
            The response containing the list of parcels.
        """
        endpoint = "/v1/vendor-parcels"
        filters = filters or OrderParcelFilter()

        params = {}
        if filters.created_at:
            params["created_at"] = filters.created_at
        if filters.cursor:
            params["cursor"] = filters.cursor
        if filters.estimate_send_at:
            params["estimate_send_at"] = filters.estimate_send_at
        if filters.ids:
            params["ids"] = filters.ids
        if filters.items_customer_ids:
            params["items.customer_ids"] = filters.items_customer_ids
        if filters.items_order_ids:
            params["items.order_ids"] = filters.items_order_ids
        if filters.items_product_ids:
            params["items.product_ids"] = filters.items_product_ids
        if filters.items_vendor_ids:
            params["items.vendor_ids"] = filters.items_vendor_ids
        if filters.per_page:
            params["per_page"] = filters.per_page
        if filters.sort:
            params["sort"] = filters.sort
        if filters.statuses:
            params["statuses"] = ",".join(str(status.value) for status in filters.statuses)

        response = self._get_sync(endpoint, params=params)
        return ParcelsResponse(**response)

    async def get_order_parcel(self, parcel_id: int) -> ParcelResponse:
        """
        Get details of a specific order parcel.

        Args:
            parcel_id: The ID of the parcel to retrieve.

        Returns:
            The response containing the parcel details.
        """
        endpoint = f"/v1/vendor-parcels/{parcel_id}"
        response = await self._get(endpoint)
        return ParcelResponse(**response)

    def get_order_parcel_sync(self, parcel_id: int) -> ParcelResponse:
        """
        Get details of a specific order parcel (synchronous version).

        Args:
            parcel_id: The ID of the parcel to retrieve.

        Returns:
            The response containing the parcel details.
        """
        endpoint = f"/v1/vendor-parcels/{parcel_id}"
        response = self._get_sync(endpoint)
        return ParcelResponse(**response)

    async def get_orders_stats(
            self,
            resource_count: ResourceStats,
            vendor_id: Optional[int] = None,
            product_id: Optional[int] = None,
            customer_id: Optional[int] = None,
            coupon_code: Optional[str] = None,
            cache_control: Optional[str] = None
    ) -> OrderStatsResponse:
        """
        Get order statistics.

        Args:
            resource_count: The type of statistics to retrieve.
            vendor_id: Optional vendor ID to filter by.
            product_id: Optional product ID to filter by.
            customer_id: Optional customer ID to filter by.
            coupon_code: Optional coupon code to filter by.
            cache_control: Optional cache control header.

        Returns:
            The response containing the order statistics.
        """
        endpoint = "/v1/orders/stats"

        params = {"resource_count": resource_count.value}
        if vendor_id is not None:
            params["vendor_id"] = vendor_id
        if product_id is not None:
            params["product_id"] = product_id
        if customer_id is not None:
            params["customer_id"] = customer_id
        if coupon_code is not None:
            params["coupon_code"] = coupon_code

        headers = {}
        if cache_control is not None:
            headers["Cache-Control"] = cache_control

        response = await self._get(endpoint, params=params, headers=headers)
        return OrderStatsResponse(**response)

    def get_orders_stats_sync(
            self,
            resource_count: ResourceStats,
            vendor_id: Optional[int] = None,
            product_id: Optional[int] = None,
            customer_id: Optional[int] = None,
            coupon_code: Optional[str] = None,
            cache_control: Optional[str] = None
    ) -> OrderStatsResponse:
        """
        Get order statistics (synchronous version).

        Args:
            resource_count: The type of statistics to retrieve.
            vendor_id: Optional vendor ID to filter by.
            product_id: Optional product ID to filter by.
            customer_id: Optional customer ID to filter by.
            coupon_code: Optional coupon code to filter by.
            cache_control: Optional cache control header.

        Returns:
            The response containing the order statistics.
        """
        endpoint = "/v1/orders/stats"

        params = {"resource_count": resource_count.value}
        if vendor_id is not None:
            params["vendor_id"] = vendor_id
        if product_id is not None:
            params["product_id"] = product_id
        if customer_id is not None:
            params["customer_id"] = customer_id
        if coupon_code is not None:
            params["coupon_code"] = coupon_code

        headers = {}
        if cache_control is not None:
            headers["Cache-Control"] = cache_control

        response = self._get_sync(endpoint, params=params, headers=headers)
        return OrderStatsResponse(**response)



"""
Tests for the Order Processing service client.
"""
import pytest

from basalam_sdk import BasalamClient
from basalam_sdk.auth import PersonalToken
from basalam_sdk.config import BasalamConfig, Environment
from basalam_sdk.order_processing.models import (
    OrderFilter,
    ItemFilter,
    OrderParcelFilter,
    ResourceStats,
    ParcelStatus
)

# Test IDs (you'll need valid IDs for testing)
TEST_ORDER_ID = 57745665
TEST_ITEM_ID = 67890
TEST_PARCEL_ID = 2196466970
TEST_VENDOR_ID = "266"
TEST_CUSTOMER_ID = "430"
TEST_PRODUCT_ID = 456


@pytest.fixture
def basalam_client():
    """Create a BasalamClient instance with real auth and config."""
    config = BasalamConfig(
        environment=Environment.PRODUCTION,
        timeout=30.0,
        user_agent="SDK-Test"
    )
    auth = PersonalToken(
        token=""
    )
    return BasalamClient(auth=auth, config=config)


# -------------------------------------------------------------------------
# Customer Orders endpoints tests
# -------------------------------------------------------------------------

@pytest.mark.asyncio
async def test_get_orders_async(basalam_client):
    """Test get_orders async method."""
    try:
        # Test without filters
        result = await basalam_client.order_processing.get_customer_orders()
        print(f"get_orders async (no filters) result: {result}")
        assert hasattr(result, 'data')

        # Test with filters
        filters = OrderFilter(
            per_page=10
        )
        result_filtered = await basalam_client.order_processing.get_customer_orders(filters=filters)
        print(f"get_orders async (with filters) result: {result_filtered}")
        assert hasattr(result_filtered, 'data')

    except Exception as e:
        print(f"get_orders async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


def test_get_customer_orders_sync(basalam_client):
    """Test get_customer_orders_sync method."""
    try:
        # Test without filters
        result = basalam_client.order_processing.get_customer_orders_sync()
        print(f"get_customer_orders_sync (no filters) result: {result}")
        assert hasattr(result, 'data')

        # Test with filters
        filters = OrderFilter(
            per_page=5,
            sort="paid_at:asc",
        )
        result_filtered = basalam_client.order_processing.get_customer_orders_sync(filters=filters)
        print(f"get_customer_orders_sync (with filters) result: {result_filtered}")
        assert hasattr(result_filtered, 'data')

    except Exception as e:
        print(f"get_customer_orders_sync error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


@pytest.mark.asyncio
async def test_get_order_async(basalam_client):
    """Test get_order async method."""
    try:
        result = await basalam_client.order_processing.get_customer_order(TEST_ORDER_ID)
        print(f"get_order async result: {result}")
        assert hasattr(result, 'id')

    except Exception as e:
        print(f"get_order async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


def test_get_customer_order_sync(basalam_client):
    """Test get_order_sync method."""
    try:
        result = basalam_client.order_processing.get_customer_order(TEST_ORDER_ID)
        print(f"get_order_sync result: {result}")
        assert hasattr(result, 'id')

    except Exception as e:
        print(f"get_order_sync error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


# -------------------------------------------------------------------------
# Customer Items endpoints tests
# -------------------------------------------------------------------------

@pytest.mark.asyncio
async def test_get_customer_order_items_async(basalam_client):
    """Test get_customer_order_items async method."""
    try:
        # Test without filters
        result = await basalam_client.order_processing.get_customer_order_items()
        print(f"get_customer_order_items async (no filters) result: {result}")
        assert hasattr(result, 'data')

        # Test with filters
        filters = ItemFilter(
            per_page=10,
            sort="created_at:desc",
            vendor_ids=[TEST_VENDOR_ID]
        )
        result_filtered = await basalam_client.order_processing.get_customer_order_items(filters=filters)
        print(f"get_customer_order_items async (with filters) result: {result_filtered}")
        assert hasattr(result_filtered, 'data')

    except Exception as e:
        print(f"get_customer_order_items async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


def test_get_customer_order_items_sync(basalam_client):
    """Test get_customer_order_items_sync method."""
    try:
        # Test without filters
        result = basalam_client.order_processing.get_customer_order_items_sync()
        print(f"get_customer_order_items_sync (no filters) result: {result}")
        assert hasattr(result, 'data')

        # Test with filters
        filters = ItemFilter(
            per_page=5,
            customer_ids=[TEST_CUSTOMER_ID],
            product_ids=[TEST_PRODUCT_ID]
        )
        result_filtered = basalam_client.order_processing.get_customer_order_items_sync(filters=filters)
        print(f"get_customer_order_items_sync (with filters) result: {result_filtered}")
        assert hasattr(result_filtered, 'data')

    except Exception as e:
        print(f"get_customer_order_items_sync error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


@pytest.mark.asyncio
async def test_get_customer_order_item_async(basalam_client):
    """Test get_customer_order_item async method."""
    try:
        result = await basalam_client.order_processing.get_customer_order_item(TEST_ORDER_ID)
        print(f"get_customer_order_item async result: {result}")
        assert hasattr(result, 'id')

    except Exception as e:
        print(f"get_customer_order_item async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


def test_get_customer_order_item_sync(basalam_client):
    """Test get_customer_order_item_sync method."""
    try:
        result = basalam_client.order_processing.get_customer_order_item_sync(TEST_ITEM_ID)
        print(f"get_customer_order_item_sync result: {result}")
        assert hasattr(result, 'id')

    except Exception as e:
        print(f"get_customer_order_item_sync error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


# -------------------------------------------------------------------------
# Vendor Parcels endpoints tests
# -------------------------------------------------------------------------

@pytest.mark.asyncio
async def test_get_vendor_orders_parcels_async(basalam_client):
    """Test get_vendor_orders_parcels async method."""
    try:
        # Test without filters
        result = await basalam_client.order_processing.get_vendor_orders_parcels()
        print(f"get_vendor_orders_parcels async (no filters) result: {result}")
        assert hasattr(result, 'data')

        # Test with filters
        filters = OrderParcelFilter(
            per_page=10,
            sort="estimate_send_at:desc",
            statuses=[ParcelStatus.NEW_ORDER, ParcelStatus.PREPARATION_IN_PROGRESS]
        )
        result_filtered = await basalam_client.order_processing.get_vendor_orders_parcels(filters=filters)
        print(f"get_vendor_orders_parcels async (with filters) result: {result_filtered}")
        assert hasattr(result_filtered, 'data')

    except Exception as e:
        print(f"get_vendor_orders_parcels async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


def test_get_vendor_orders_parcels_sync(basalam_client):
    """Test get_vendor_orders_parcels_sync method."""
    try:
        # Test without filters
        result = basalam_client.order_processing.get_vendor_orders_parcels_sync()
        print(f"get_vendor_orders_parcels_sync (no filters) result: {result}")
        assert hasattr(result, 'data')

        # Test with filters
        filters = OrderParcelFilter(
            per_page=5
        )
        result_filtered = basalam_client.order_processing.get_vendor_orders_parcels_sync(filters=filters)
        print(f"get_vendor_orders_parcels_sync (with filters) result: {result_filtered}")
        assert hasattr(result_filtered, 'data')

    except Exception as e:
        print(f"get_vendor_orders_parcels_sync error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


@pytest.mark.asyncio
async def test_get_order_parcel_async(basalam_client):
    """Test get_order_parcel async method."""
    try:
        result = await basalam_client.order_processing.get_order_parcel(TEST_PARCEL_ID)
        print(f"get_order_parcel async result: {result}")
        assert hasattr(result, 'id')

    except Exception as e:
        print(f"get_order_parcel async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


def test_get_order_parcel_sync(basalam_client):
    """Test get_order_parcel_sync method."""
    try:
        result = basalam_client.order_processing.get_order_parcel_sync(TEST_PARCEL_ID)
        print(f"get_order_parcel_sync result: {result}")
        assert hasattr(result, 'id')

    except Exception as e:
        print(f"get_order_parcel_sync error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


# -------------------------------------------------------------------------
# Order Statistics endpoints tests
# -------------------------------------------------------------------------

@pytest.mark.asyncio
async def test_get_orders_stats_async(basalam_client):
    """Test get_orders_stats async method."""
    try:
        # Test with minimal parameters
        result = basalam_client.order_processing.get_orders_stats_sync(
            resource_count=ResourceStats.NUMBER_OF_NOT_SHIPPED_ORDERS_PER_VENDOR,
            vendor_id=TEST_VENDOR_ID
        )
        print(f"get_orders_stats_sync (minimal) result: {result}")
        assert hasattr(result, 'result')

        # Test with vendor-specific stats
        result_vendor = basalam_client.order_processing.get_orders_stats_sync(
            resource_count=ResourceStats.NUMBER_OF_COMPLETED_ORDERS_PER_VENDOR,
            vendor_id=TEST_VENDOR_ID
        )
        print(f"get_orders_stats async (full) result: {result_vendor}")
        assert hasattr(result_vendor, 'result')

    except Exception as e:
        print(f"get_orders_stats async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


def test_get_orders_stats_sync(basalam_client):
    """Test get_orders_stats_sync method."""
    try:
        # Test with minimal parameters
        result = basalam_client.order_processing.get_orders_stats_sync(
            resource_count=ResourceStats.NUMBER_OF_NOT_SHIPPED_ORDERS_PER_VENDOR,
            vendor_id=TEST_VENDOR_ID
        )
        print(f"get_orders_stats_sync (minimal) result: {result}")
        assert hasattr(result, 'result')

        # Test with vendor-specific stats
        result_vendor = basalam_client.order_processing.get_orders_stats_sync(
            resource_count=ResourceStats.NUMBER_OF_COMPLETED_ORDERS_PER_VENDOR,
            vendor_id=TEST_VENDOR_ID
        )
        print(f"get_orders_stats_sync (vendor) result: {result_vendor}")
        assert hasattr(result_vendor, 'result')

    except Exception as e:
        print(f"get_orders_stats_sync error: {e}")
        # Don't fail the test for API errors, just log them
        assert True



"""
Basalam Python SDK for accessing the Basalam API.

This package provides a clean and simple interface to interact with Basalam's microservices.
"""

from .auth import BaseAuth, ClientCredentials, AuthorizationCode, PersonalToken, Scope
from .basalam_client import BasalamClient
from .base_client import BaseClient
from .chat.client import ChatService
from .config import BasalamConfig
# Import service clients
from .core.client import CoreService
from .errors import BasalamError, BasalamAPIError, BasalamAuthError, BasalamValidationError
from .order.client import OrderService
from .order_processing.client import OrderProcessingService
from .search.client import SearchService
from .upload.client import UploadService
from .version import __version__, __sdk_name__, get_user_agent
from .wallet.client import WalletService
from .webhook.client import WebhookService

__all__ = [
    # Main client
    "BasalamClient",
    "BaseClient",

    # Authentication
    "BaseAuth",
    "ClientCredentials",
    "AuthorizationCode",
    "PersonalToken",
    "Scope",

    # Configuration
    "BasalamConfig",

    # Errors
    "BasalamError",
    "BasalamAPIError",
    "BasalamAuthError",
    "BasalamValidationError",

    # Service clients
    "CoreService",
    "WalletService",
    "OrderService",
    "OrderProcessingService",
    "SearchService",
    "UploadService",
    "ChatService",
    "WebhookService",

    # Version and utilities
    "__version__",
    "__sdk_name__",
    "get_user_agent",
]



"""
Wallet service client for the Basalam SDK.

This module provides access to Basalam's wallet service APIs.
"""

from .client import WalletService
from .models import (
    BalanceFilter,
    CreditResponse,
    CreditTypeResponse,
    HistoryCreditItemResponse,
    HistoryItemResponse,
    HistoryPaginationResponse,
    HistorySpendItemResponse,
    HistorySpendResponse,
    NewHistoryCreditResponse,
    ReasonResponse,
    ReferenceRequest,
    ReferenceResponse,
    SpendCreditRequest,
    SpendItemResponse,
    SpendResponse,
)

__all__ = [
    "WalletService",
    "BalanceFilter",
    "CreditResponse",
    "CreditTypeResponse",
    "HistoryCreditItemResponse",
    "HistoryItemResponse",
    "HistoryPaginationResponse",
    "HistorySpendItemResponse",
    "HistorySpendResponse",
    "NewHistoryCreditResponse",
    "ReasonResponse",
    "ReferenceRequest",
    "ReferenceResponse",
    "SpendCreditRequest",
    "SpendItemResponse",
    "SpendResponse",
]



"""
Webhook service module for the Basalam SDK.

This module provides access to Basalam's webhook service APIs.
"""

from .client import WebhookService
from .models import (
    ServiceResource,
    ServiceListResource,
    CreateServiceRequest,
    WebhookResource,
    WebhookListResource,
    CreateWebhookRequest,
    EventResource,
    EventListResource,
    UpdateWebhookRequest,
    DeleteWebhookResponse,
    WebhookLogResource,
    WebhookLogListResource,
    RegisterClientRequest,
    UnRegisterClientRequest,
    UnRegisterClientResponse,
    ClientResource,
    ClientListResource,
    WebhookRegisteredOnResource,
    WebhookRegisteredOnListResource,
)

__all__ = [
    "WebhookService",
    "ServiceResource",
    "ServiceListResource",
    "CreateServiceRequest",
    "WebhookResource",
    "WebhookListResource",
    "CreateWebhookRequest",
    "EventResource",
    "EventListResource",
    "UpdateWebhookRequest",
    "DeleteWebhookResponse",
    "WebhookLogResource",
    "WebhookLogListResource",
    "RegisterClientRequest",
    "UnRegisterClientRequest",
    "UnRegisterClientResponse",
    "ClientResource",
    "ClientListResource",
    "WebhookRegisteredOnResource",
    "WebhookRegisteredOnListResource",
]



"""
Models for the Basalam Wallet Service.

This module contains data models for the Wallet Service API.
"""

from datetime import datetime
from typing import Dict, List, Optional

from pydantic import BaseModel


class BalanceFilter(BaseModel):
    """Filter for balance requests."""
    cash: Optional[bool] = None
    settleable: Optional[bool] = None
    vendor: Optional[bool] = None
    customer: Optional[bool] = None


class ReasonResponse(BaseModel):
    """Reason response model."""
    id: int
    description: str


class ReferenceResponse(BaseModel):
    """Reference response model."""
    reference_type_id: int
    title: str
    slug: str
    reference_id: int


class ReferenceRequest(BaseModel):
    """Reference response model."""
    reference_type_id: int
    reference_id: int


class CreditTypeResponse(BaseModel):
    """Credit type response model."""
    id: int
    title: str
    parent: Optional['CreditTypeResponse'] = None


class SpendCreditRequest(BaseModel):
    """Spend credit request model."""
    reason_id: int
    reference_id: int
    amount: int
    description: str
    types: Optional[List[int]] = None
    settleable: Optional[bool] = None
    references: Optional[Dict[str, int]] = None


class CreditResponse(BaseModel):
    """Credit response model."""
    id: int
    created_at: datetime
    updated_at: datetime
    expire_at: Optional[datetime] = None
    user_id: int
    client_id: Optional[int] = None
    reference_id: Optional[int] = None
    reason: Optional[ReasonResponse] = None
    amount: int
    remained_amount: int
    description: Optional[str] = None
    type: CreditTypeResponse
    references: Optional[List[ReferenceResponse]] = None


class SpendItemResponse(BaseModel):
    """Spend item response model."""
    id: int
    amount: int
    references: List[ReferenceResponse]
    credit: CreditResponse


class SpendResponse(BaseModel):
    """Spend response model."""
    id: Optional[int] = None
    created_at: datetime
    updated_at: datetime
    deleted_at: Optional[datetime] = None
    amount: int
    description: Optional[str] = None
    user_id: int
    client_id: Optional[int] = None
    reference_id: int
    reason: ReasonResponse
    rollback_reason: ReasonResponse
    items: List[SpendItemResponse]
    references: List[ReferenceResponse]


class HistoryCreditItemResponse(BaseModel):
    """History credit item response model."""
    id: int
    amount: int
    remained_amount: int
    created_at: datetime
    expire_at: Optional[datetime] = None
    type: CreditTypeResponse


class NewHistoryCreditResponse(BaseModel):
    """New history credit response model."""
    amount: Optional[int] = None
    remained_amount: Optional[int] = None
    created_at: Optional[datetime] = None
    items: Optional[List[HistoryCreditItemResponse]] = None


class HistorySpendItemResponse(BaseModel):
    """History spend item response model."""
    id: int
    amount: int


class HistorySpendResponse(BaseModel):
    """History spend response model."""
    id: Optional[int] = None
    created_at: Optional[datetime] = None
    amount: Optional[int] = None
    items: Optional[List[HistorySpendItemResponse]] = None


class HistoryItemResponse(BaseModel):
    """History item response model."""
    time: datetime
    amount: int
    subtotal: int
    description: str
    main_reference_id: int
    reason: Optional[ReasonResponse] = None
    references: List[ReferenceResponse]
    related_credit: Optional[NewHistoryCreditResponse] = None
    related_spend: Optional[HistorySpendResponse] = None


class HistoryPaginationResponse(BaseModel):
    """History pagination response model."""
    data: List[HistoryItemResponse]
    total: int
    per_page: int
    current_page: int
    last_page: int
    from_: int
    to: int



"""
Chat service module for the Basalam SDK.

This module provides access to Basalam's chat service APIs.
"""

from .client import ChatService
from .models import (
    Attachment,
    AttachmentFile,
    ChatListData,
    ChatListResponse,
    ChatResponse,
    Contact,
    CreateChatRequest,
    GetChatsRequest,
    GetMessagesRequest,
    GetMessagesResponse,
    MessageContent,
    MessageFile,
    MessageInput,
    MessageLink,
    MessageRequest,
    MessageSender,
    MessageTypeEnum,
    MessageOrderByEnum,
    MessageFiltersEnum,
    EditMessageRequest,
    EditMessageResponse,
    DeleteMessageRequest,
    DeleteChatsRequest,
    ForwardMessageRequest,
    BooleanResponse,
    UnseenChatCountResponse,
    BotApiResponse,
)

__all__ = [
    "ChatService",
    "Attachment",
    "AttachmentFile",
    "ChatListData",
    "ChatListResponse",
    "ChatResponse",
    "Contact",
    "CreateChatRequest",
    "GetChatsRequest",
    "GetMessagesRequest",
    "GetMessagesResponse",
    "MessageContent",
    "MessageFile",
    "MessageInput",
    "MessageLink",
    "MessageRequest",
    "MessageSender",
    "MessageTypeEnum",
    "MessageOrderByEnum",
    "MessageFiltersEnum",
    "EditMessageRequest",
    "EditMessageResponse",
    "DeleteMessageRequest",
    "DeleteChatsRequest",
    "ForwardMessageRequest",
    "BooleanResponse",
    "UnseenChatCountResponse",
    "BotApiResponse",
]



"""
Client for the Search service API.

This module provides a client for interacting with Basalam's search service.
"""

import logging
from typing import Dict, Any

from .models import ProductSearchModel
from ..base_client import BaseClient

logger = logging.getLogger(__name__)


class SearchService(BaseClient):
    """Client for the Search service API."""

    def __init__(self, **kwargs):
        """Initialize the search service client."""
        super().__init__(service="search", **kwargs)

    async def search_products(self, request: ProductSearchModel) -> Dict[str, Any]:
        """
        Search for products.
        
        Args:
            request: The search request model containing filters and search parameters.
            
        Returns:
            The search results.
        """
        endpoint = "/v1/products/search"
        response = await self._post(endpoint, json_data=request.model_dump(exclude_none=True), require_auth=False)
        return response

    def search_products_sync(self, request: ProductSearchModel) -> Dict[str, Any]:
        """
        Search for products (synchronous version).
        
        Args:
            request: The search request model containing filters and search parameters.
            
        Returns:
            The search results.
        """
        endpoint = "/v1/products/search"
        response = self._post_sync(endpoint, json_data=request.model_dump(exclude_none=True), require_auth=False)
        return response



"""
Tests for the Upload service client.
"""

import pytest

from basalam_sdk import BasalamClient
from basalam_sdk.auth import PersonalToken
from basalam_sdk.config import BasalamConfig, Environment


@pytest.fixture
def basalam_client():
    """Create a BasalamClient instance with real auth and config."""
    config = BasalamConfig(
        environment=Environment.PRODUCTION,
        timeout=30.0,
        user_agent="SDK-Test"
    )
    auth = PersonalToken(
        token=""
    )
    return BasalamClient(auth=auth, config=config)


# -------------------------------------------------------------------------
# Upload endpoints tests
# -------------------------------------------------------------------------

@pytest.mark.asyncio
async def test_upload_file_async(basalam_client):
    """Test upload_file async method."""
    try:
        with open('test1.png', "rb") as file_data:
            result = await basalam_client.upload.upload_file(
                file=file_data,
                file_type="product.photo",
                custom_unique_name="test-file-async",
                expire_minutes=60
            )
            print(f"upload_file async result: {result}")
            assert result is not None
            assert hasattr(result, 'id')
            assert hasattr(result, 'file_name')
            assert hasattr(result, 'urls')
    except Exception as e:
        print(f"upload_file async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


def test_upload_file_sync(basalam_client):
    """Test upload_file_sync method."""
    try:
        with open('test1.png', "rb") as file_data:
            result = basalam_client.upload.upload_file_sync(
                file=file_data,
                file_type='product.photo',
                custom_unique_name="test-file-sync",
                expire_minutes=60
            )
            print(f"upload_file_sync result: {result}")
            assert result is not None
            assert hasattr(result, 'id')
            assert hasattr(result, 'file_name')
            assert hasattr(result, 'urls')
    except Exception as e:
        print(f"upload_file_sync error: {e}")
        # Don't fail the test for API errors, just log them
        assert True



"""
Models for the Chat service API.
"""
from enum import Enum
from typing import Dict, List, Optional, Any

from pydantic import BaseModel


class MessageTypeEnum(str, Enum):
    """Message type enumeration."""
    FILE = "file"
    OLD_FILE = "old_file"
    PRODUCT = "product"
    VENDOR = "vendor"
    SOCIAL_POST = "social_post"
    USER = "user"
    COUPON = "coupon"
    NOTIFICATION = "notification"
    ORDER_PROCESS = "order_process"
    STORY = "story"
    PRODUCT_VOICE = "product_voice"
    REVIEW = "review"
    ORDER = "order"
    TEXT = "text"
    PICTURE = "picture"
    GALLERY = "gallery"
    LABEL = "label"
    STICKER = "sticker"
    VOICE = "voice"
    VIDEO = "video"
    LOCATION = "location"


class MessageOrderByEnum(str, Enum):
    """Order by enumeration for get_chats endpoint."""
    UPDATED_AT = "updated_at"
    MODIFIED_AT = "modified_at"


class MessageFiltersEnum(str, Enum):
    """Filters enumeration for get_chats endpoint."""
    UNSEEN = "unseen"
    ORDER = "order"


class MessageInput(BaseModel):
    """Message input model."""
    text: Optional[str] = None
    entity_id: Optional[int] = None


class AttachmentFile(BaseModel):
    """Attachment file model."""
    id: int
    url: str
    height: Optional[int] = None
    width: Optional[int] = None
    name: Optional[str] = None
    type: Optional[str] = None
    size: Optional[int] = None
    blur_hash: Optional[str] = None


class Attachment(BaseModel):
    """Attachment model."""
    files: Optional[List[AttachmentFile]] = None


class MessageRequest(BaseModel):
    """Message request model."""
    chat_id: int
    content: Optional[MessageInput] = None
    message_type: MessageTypeEnum
    attachment: Optional[Attachment] = None
    replied_message_id: Optional[int] = None
    message_metadata: Optional[Dict[str, Any]] = None
    temp_id: Optional[int] = None


class GetChatsRequest(BaseModel):
    """Get chats request model."""
    limit: Optional[int] = 30
    order_by: Optional[MessageOrderByEnum] = MessageOrderByEnum.UPDATED_AT
    updated_from: Optional[str] = None
    updated_before: Optional[str] = None
    modified_from: Optional[str] = None
    modified_before: Optional[str] = None
    filters: Optional[MessageFiltersEnum] = None


class LocationResource(BaseModel):
    """Location request model."""
    geo_width: int
    geo_height: int


class MessageLink(BaseModel):
    """Message link model."""
    url: str
    start_index: int
    end_index: int
    is_basalam_link: bool


class MessageFile(BaseModel):
    """Message file model."""
    id: int
    url: str
    name: Optional[str] = None
    size: Optional[int] = None
    type: Optional[str] = None
    width: Optional[int] = None
    height: Optional[int] = None
    blur_hash: Optional[str] = None


class MessageContent(BaseModel):
    """Message content model."""
    links: Optional[List[MessageLink]] = []
    files: Optional[List[MessageFile]] = []
    text: Optional[str] = None
    entity_id: Optional[int] = None
    location: Optional[LocationResource] = None


class MessageSender(BaseModel):
    """Message sender model."""
    id: str


class BaseMessageResource(BaseModel):
    """Base message resource model."""
    id: int
    chat_id: int
    seen_at: str
    created_at: str
    updated_at: str
    message_type: str
    sender: MessageSender
    content: MessageContent


class MessageResource(BaseModel):
    """Message resource model."""
    id: int
    chat_id: int
    seen_at: Optional[str] = None
    created_at: str
    updated_at: str
    message_type: str
    sender: MessageSender  # also could be GroupSender
    content: MessageContent
    replied_message: Optional[BaseMessageResource] = None


class MessageResponse(BaseModel):
    """Message response model for create_message endpoint."""
    data: MessageResource
    temp_id: Optional[int] = None


class CreateChatRequest(BaseModel):
    """Create chat request model."""
    user_id: Optional[int] = None
    hash_id: Optional[str] = None


class GroupMetadata(BaseModel):
    """Group model."""
    title: str
    description: Optional[str] = None
    avatar: Optional[str] = None
    link: str
    creator_id: int
    id: int
    chat_id: int


class ChannelMetadata(BaseModel):
    """Channel metadata model."""
    title: str
    description: Optional[str] = None
    avatar: Optional[str] = None
    link: Optional[str] = None
    owner: int
    is_admin: bool = False
    members_count: Optional[int] = None
    can_leave: bool = True
    verified: bool = True


class ChatResource(BaseModel):
    """Chat resource model for create_chat response."""
    id: int
    marked_as_unread: bool
    unseen_message_count: int
    updated_at: str
    deleted_at: Optional[str] = None
    created_at: Optional[str] = None
    modified_at: Optional[str] = None
    chat_type: str
    last_seen_id: Optional[int] = None
    last_message: Optional[MessageResource] = None
    contact: Optional[Dict[str, Any]] = None
    contact_id: Optional[int] = None
    contact_is_blocked: bool
    show_approvals: bool
    reply_markup: Optional[Dict[str, Any]] = None
    archive_state: Optional[str] = None
    group: Optional[GroupMetadata] = None
    channel: Optional[ChannelMetadata] = None


class CreateChatResponse(BaseModel):
    """Chat response model for create_chat endpoint."""
    data: ChatResource


class GetMessagesRequest(BaseModel):
    """Get messages request model."""
    chat_id: int
    message_id: Optional[int] = None
    limit: Optional[int] = 20
    order: Optional[str] = "desc"  # "desc" or "asc"
    cmp: Optional[str] = "lt"  # "lte", "lt", "gte", "gt", "bt"


class GetMessagesListData(BaseModel):
    """Get messages list data model."""
    messages: List[MessageResource]


class GetMessagesResponse(BaseModel):
    """Get messages response model."""
    data: GetMessagesListData


class Contact(BaseModel):
    """Contact model."""
    id: int
    hash_id: str
    name: str
    avatar: Optional[str] = None
    badge: Optional[str] = None
    verified: bool
    vendor: Optional[Dict[str, Any]] = None
    description: Optional[str] = None


class ChatResponse(BaseModel):
    """Response model for a single chat."""
    id: int
    marked_as_unread: bool
    unseen_message_count: int
    updated_at: str
    deleted_at: Optional[str] = None
    created_at: Optional[str] = None
    modified_at: Optional[str] = None
    chat_type: str
    last_seen_id: Optional[int] = None
    last_message: Optional[MessageResource] = None
    contact: Optional[Contact] = None
    contact_id: Optional[int] = None
    contact_is_blocked: bool
    show_approvals: bool
    reply_markup: Optional[Dict[str, Any]] = None
    archive_state: Optional[str] = None
    group: Optional[GroupMetadata] = None
    channel: Optional[ChannelMetadata] = None


class ChatListData(BaseModel):
    """Chat list data model."""
    chats: List[ChatResponse]


class ChatListResponse(BaseModel):
    """Response model for chat list endpoint."""
    data: ChatListData


class EditMessageRequest(BaseModel):
    """Edit message request model."""
    message_id: int
    content: Optional[MessageInput] = None


class DeleteMessageRequest(BaseModel):
    """Delete message request model."""
    message_ids: List[int]


class DeleteChatsRequest(BaseModel):
    """Delete chats request model."""
    chat_ids: List[int]


class ForwardMessageRequest(BaseModel):
    """Forward message request model."""
    message_ids: List[int]
    chat_ids: List[int]


class BooleanResponse(BaseModel):
    """Boolean response model."""
    data: bool


class UnseenChatCountResponse(BaseModel):
    """Unseen chat count response model."""
    count: int
    more_than_count: bool


class EditMessageResponse(BaseModel):
    """Edit message response model."""
    data: MessageResource


class BotApiResponse(BaseModel):
    """Bot API response model."""
    ok: bool
    result: Optional[Any] = None
    description: Optional[str] = None
    error_code: Optional[int] = None



"""
Tests for the Chat service client.
"""
import pytest

from basalam_sdk import BasalamClient
from basalam_sdk.auth import PersonalToken
from basalam_sdk.chat.models import (
    MessageRequest,
    CreateChatRequest,
    MessageInput,
    MessageTypeEnum,
    GetMessagesRequest,
    GetChatsRequest,
    MessageOrderByEnum,
    EditMessageRequest,
    DeleteMessageRequest,
    DeleteChatsRequest,
    ForwardMessageRequest,
    BotApiResponse,
)
from basalam_sdk.config import BasalamConfig, Environment

# Test data
TEST_CHAT_ID = 183583802
TEST_USER_ID = 430
TEST_BOT_TOKEN = ""


@pytest.fixture
def basalam_client():
    """Create a BasalamClient instance with real auth and config."""
    config = BasalamConfig(
        environment=Environment.PRODUCTION,
        timeout=30.0,
        user_agent="SDK-Test"
    )
    auth = PersonalToken(
        token=""
    )
    return BasalamClient(auth=auth, config=config)


# -------------------------------------------------------------------------
# Message endpoints tests
# -------------------------------------------------------------------------

@pytest.mark.asyncio
async def test_create_message_async(basalam_client):
    """Test create_message async method."""
    try:
        message_input = MessageInput(
            text="Test message",
            entity_id=123
        )
        request = MessageRequest(
            chat_id=TEST_CHAT_ID,
            content=message_input,
            message_type=MessageTypeEnum.TEXT,
            temp_id=12345
        )
        result = await basalam_client.chat.create_message(
            request=request,
        )
        print(f"create_message async result: {result}")
        assert result is not None
        assert hasattr(result, 'data')
        assert hasattr(result.data, 'id')
    except Exception as e:
        print(f"create_message async error: {e}")
        assert True


def test_create_message_sync(basalam_client):
    """Test create_message_sync method."""
    try:
        message_input = MessageInput(
            text="Test message",
            entity_id=123
        )
        request = MessageRequest(
            chat_id=TEST_CHAT_ID,
            content=message_input,
            message_type=MessageTypeEnum.TEXT,
            temp_id=12345
        )
        result = basalam_client.chat.create_message_sync(
            request=request
        )
        print(f"create_message_sync result: {result}")
        assert result is not None
        assert hasattr(result, 'data')
        assert hasattr(result.data, 'id')
    except Exception as e:
        print(f"create_message_sync error: {e}")
        assert True


@pytest.mark.asyncio
async def test_get_messages_async(basalam_client):
    """Test get_messages async method."""
    try:
        request = GetMessagesRequest(
            chat_id=TEST_CHAT_ID
        )
        result = await basalam_client.chat.get_messages(
            request=request
        )
        print(f"get_messages async result: {result}")
    except Exception as e:
        print(f"get_messages async error: {e}")
        assert True


def test_get_messages_sync(basalam_client):
    """Test get_messages_sync method."""
    try:
        request = GetMessagesRequest(
            chat_id=TEST_CHAT_ID,

        )
        result = basalam_client.chat.get_messages_sync(
            request=request
        )
        print(f"get_messages_sync result: {result}")
        assert result is not None
        assert hasattr(result, 'data')
        assert hasattr(result.data, 'messages')
        assert isinstance(result.data.messages, list)
    except Exception as e:
        print(f"get_messages_sync error: {e}")
        assert True


# -------------------------------------------------------------------------
# Chat endpoints tests
# -------------------------------------------------------------------------

@pytest.mark.asyncio
async def test_create_chat_async(basalam_client):
    """Test create_chat async method."""
    try:
        request = CreateChatRequest(
            user_id=1308962
        )
        result = await basalam_client.chat.create_chat(
            request=request
        )
        print(f"create_chat async result: {result}")
        assert result is not None
        assert hasattr(result, 'data')
        assert hasattr(result.data, 'id')
    except Exception as e:
        print(f"create_chat async error: {e}")
        assert True


def test_create_chat_sync(basalam_client):
    """Test create_chat_sync method."""
    try:
        request = CreateChatRequest(
            user_id=1308962
        )
        result = basalam_client.chat.create_chat_sync(
            request=request
        )
        print(f"create_chat_sync result: {result}")
        assert result is not None
        assert hasattr(result, 'data')
        assert hasattr(result.data, 'id')
    except Exception as e:
        print(f"create_chat_sync error: {e}")
        assert True


@pytest.mark.asyncio
async def test_get_chats_async(basalam_client):
    """Test get_chats async method."""
    try:
        request = GetChatsRequest(
            limit=10
        )
        result = await basalam_client.chat.get_chats(
            request=request
        )
        print(f"get_chats async result: {result}")
        assert result is not None
        assert hasattr(result, 'data')
        assert hasattr(result.data, 'chats')
        assert isinstance(result.data.chats, list)
    except Exception as e:
        print(f"get_chats async error: {e}")
        assert True


def test_get_chats_sync(basalam_client):
    """Test get_chats_sync method."""
    try:
        request = GetChatsRequest(
            limit=10,
            order_by=MessageOrderByEnum.UPDATED_AT
        )
        result = basalam_client.chat.get_chats_sync(
            request=request
        )
        print(f"get_chats_sync result: {result}")
        assert result is not None
        assert hasattr(result, 'data')
        assert hasattr(result.data, 'chats')
        assert isinstance(result.data.chats, list)
    except Exception as e:
        print(f"get_chats_sync error: {e}")
        assert True


# -------------------------------------------------------------------------
# Model dump exclude none tests
# -------------------------------------------------------------------------

@pytest.mark.asyncio
async def test_model_dump_exclude_none_async(basalam_client):
    """Test that model_dump(exclude_none=True) works correctly for chat models."""
    chat_service = basalam_client.chat

    # Create a message request with optional fields set to None
    message_input = MessageInput(
        text="Test message",
        entity_id=None  # This should be excluded from the request
    )
    request = MessageRequest(
        chat_id=TEST_CHAT_ID,
        message_type=MessageTypeEnum.TEXT,
        message_source=None,  # This should be excluded from the request
        message=message_input,
        attachment=None,  # This should be excluded from the request
        replied_message_id=None,  # This should be excluded from the request
        message_metadata=None,  # This should be excluded from the request
        temp_id=None  # This should be excluded from the request
    )

    # Test the model_dump method
    dumped_data = request.model_dump(exclude_none=True)
    print(f"Model dump result: {dumped_data}")

    # Verify that None values are excluded
    assert "message_source" not in dumped_data
    assert "attachment" not in dumped_data
    assert "replied_message_id" not in dumped_data
    assert "message_metadata" not in dumped_data
    assert "temp_id" not in dumped_data

    # Verify that required fields are included
    assert "chat_id" in dumped_data
    assert "message_type" in dumped_data
    assert "message" in dumped_data

    # Verify that nested None values are excluded
    assert "entity_id" not in dumped_data["message"]
    assert "text" in dumped_data["message"]


def test_model_dump_exclude_none_sync(basalam_client):
    """Test that model_dump(exclude_none=True) works correctly for chat models (sync version)."""
    chat_service = basalam_client.chat

    # Create a chat request with optional fields set to None
    request = CreateChatRequest(
        chat_type="private",
        user_id=None,  # This should be excluded from the request
        hash_id=None  # This should be excluded from the request
    )

    # Test the model_dump method
    dumped_data = request.model_dump(exclude_none=True)
    print(f"Model dump result: {dumped_data}")

    # Verify that None values are excluded
    assert "user_id" not in dumped_data
    assert "hash_id" not in dumped_data

    # Verify that required fields are included
    assert "chat_type" in dumped_data


# -------------------------------------------------------------------------
# Edit message endpoints tests
# -------------------------------------------------------------------------

@pytest.mark.asyncio
async def test_edit_message_async(basalam_client):
    """Test edit_message async method."""

    try:
        message_input = MessageInput(
            text="Updated test message"
        )
        request = EditMessageRequest(
            message_id=980466407,
            content=message_input
        )
        result = await basalam_client.chat.edit_message(
            request=request
        )
        print(f"edit_message async result: {result}")
        assert result is not None
        assert hasattr(result, 'data')
        assert hasattr(result.data, 'id')
    except Exception as e:
        print(f"edit_message async error: {e}")
        assert True


def test_edit_message_sync(basalam_client):
    """Test edit_message_sync method."""

    try:
        request = EditMessageRequest(
            message_id=980466407,
            content=MessageInput(
                text="Updated twice",
                entity_id=2
        )
        )
        result = basalam_client.chat.edit_message_sync(
            request=request
        )
        print(f"edit_message_sync result: {result}")
        assert result is not None
        assert hasattr(result, 'data')
        assert hasattr(result.data, 'id')
    except Exception as e:
        print(f"edit_message_sync error: {e}")
        assert True


# -------------------------------------------------------------------------
# Delete message endpoints tests
# -------------------------------------------------------------------------

@pytest.mark.asyncio
async def test_delete_message_async(basalam_client):
    """Test delete_message async method."""

    try:
        request = DeleteMessageRequest(
            message_ids=[980466407]
        )
        result = await basalam_client.chat.delete_message(
            request=request
        )
        print(f"delete_message async result: {result}")
        assert result is not None
        assert hasattr(result, 'data')
        assert isinstance(result.data, bool)
    except Exception as e:
        print(f"delete_message async error: {e}")
        assert True


def test_delete_message_sync(basalam_client):
    """Test delete_message_sync method."""

    try:
        request = DeleteMessageRequest(
            message_ids=[123456, 123457]
        )
        result = basalam_client.chat.delete_message_sync(
            request=request
        )
        print(f"delete_message_sync result: {result}")
        assert result is not None
        assert hasattr(result, 'data')
        assert isinstance(result.data, bool)
    except Exception as e:
        print(f"delete_message_sync error: {e}")
        assert True


# -------------------------------------------------------------------------
# Delete chats endpoints tests
# -------------------------------------------------------------------------

@pytest.mark.asyncio
async def test_delete_chats_async(basalam_client):
    """Test delete_chats async method."""

    try:
        request = DeleteChatsRequest(
            chat_ids=[TEST_CHAT_ID]
        )
        result = await basalam_client.chat.delete_chats(
            request=request
        )
        print(f"delete_chats async result: {result}")
        assert result is not None
        assert hasattr(result, 'data')
        assert isinstance(result.data, bool)
    except Exception as e:
        print(f"delete_chats async error: {e}")
        assert True


def test_delete_chats_sync(basalam_client):
    """Test delete_chats_sync method."""

    try:
        request = DeleteChatsRequest(
            chat_ids=[123456, 123457]
        )
        result = basalam_client.chat.delete_chats_sync(
            request=request
        )
        print(f"delete_chats_sync result: {result}")
        assert result is not None
        assert hasattr(result, 'data')
        assert isinstance(result.data, bool)
    except Exception as e:
        print(f"delete_chats_sync error: {e}")
        assert True


# -------------------------------------------------------------------------
# Forward message endpoints tests
# -------------------------------------------------------------------------

@pytest.mark.asyncio
async def test_forward_message_async(basalam_client):
    """Test forward_message async method."""

    try:
        request = ForwardMessageRequest(
            message_ids=[983365122, 983365104],
            chat_ids=[TEST_CHAT_ID]
        )
        result = await basalam_client.chat.forward_message(
            request=request
        )
        print(f"forward_message async result: {result}")
        assert result is not None
        assert hasattr(result, 'data')
        assert isinstance(result.data, bool)
    except Exception as e:
        print(f"forward_message async error: {e}")
        assert True


def test_forward_message_sync(basalam_client):
    """Test forward_message_sync method."""

    try:
        request = ForwardMessageRequest(
            message_ids=[980484321],
            chat_ids=[TEST_CHAT_ID]
        )
        result = basalam_client.chat.forward_message_sync(
            request=request
        )
        print(f"forward_message_sync result: {result}")
        assert result is not None
        assert hasattr(result, 'data')
        assert isinstance(result.data, bool)
    except Exception as e:
        print(f"forward_message_sync error: {e}")
        assert True


# -------------------------------------------------------------------------
# Get unseen chat count endpoints tests
# -------------------------------------------------------------------------

@pytest.mark.asyncio
async def test_get_unseen_chat_count_async(basalam_client):
    """Test get_unseen_chat_count async method."""
    try:
        result = await basalam_client.chat.get_unseen_chat_count()
        print(f"get_unseen_chat_count async result: {result}")
        assert result is not None

    except Exception as e:
        print(f"get_unseen_chat_count async error: {e}")
        assert True


def test_get_unseen_chat_count_sync(basalam_client):
    """Test get_unseen_chat_count_sync method."""
    try:
        result = basalam_client.chat.get_unseen_chat_count_sync()
        print(f"get_unseen_chat_count_sync result: {result}")
        assert result is not None

    except Exception as e:
        print(f"get_unseen_chat_count_sync error: {e}")
        assert True


# -------------------------------------------------------------------------
# Bot endpoints tests - getWebhookInfo
# -------------------------------------------------------------------------

@pytest.mark.asyncio
async def test_get_webhook_info_async(basalam_client):
    """Test get_webhook_info async method (GET)."""
    try:
        result = await basalam_client.chat.get_webhook_info(
            token=TEST_BOT_TOKEN
        )
        print(f"get_webhook_info async result: {result}")
        assert result is not None
        assert hasattr(result, 'ok')
        assert isinstance(result.ok, bool)
    except Exception as e:
        print(f"get_webhook_info async error: {e}")
        assert True


def test_get_webhook_info_sync(basalam_client):
    """Test get_webhook_info_sync method (GET)."""
    try:
        result = basalam_client.chat.get_webhook_info_sync(
            token=TEST_BOT_TOKEN
        )
        print(f"get_webhook_info_sync result: {result}")
        assert result is not None
        assert hasattr(result, 'ok')
        assert isinstance(result.ok, bool)
    except Exception as e:
        print(f"get_webhook_info_sync error: {e}")
        assert True


@pytest.mark.asyncio
async def test_get_webhook_info_post_async(basalam_client):
    """Test get_webhook_info_post async method (POST)."""
    try:
        result = await basalam_client.chat.get_webhook_info_post(
            token=TEST_BOT_TOKEN
        )
        print(f"get_webhook_info_post async result: {result}")
        assert result is not None
        assert hasattr(result, 'ok')
        assert isinstance(result.ok, bool)
    except Exception as e:
        print(f"get_webhook_info_post async error: {e}")
        assert True


def test_get_webhook_info_post_sync(basalam_client):
    """Test get_webhook_info_post_sync method (POST)."""
    try:
        result = basalam_client.chat.get_webhook_info_post_sync(
            token=TEST_BOT_TOKEN
        )
        print(f"get_webhook_info_post_sync result: {result}")
        assert result is not None
        assert hasattr(result, 'ok')
        assert isinstance(result.ok, bool)
    except Exception as e:
        print(f"get_webhook_info_post_sync error: {e}")
        assert True


# -------------------------------------------------------------------------
# Bot endpoints tests - logOut
# -------------------------------------------------------------------------

@pytest.mark.asyncio
async def test_log_out_async(basalam_client):
    """Test log_out async method (GET)."""
    try:
        result = await basalam_client.chat.log_out(
            token=TEST_BOT_TOKEN
        )
        print(f"log_out async result: {result}")
        assert result is not None
        assert hasattr(result, 'ok')
        assert isinstance(result.ok, bool)
    except Exception as e:
        print(f"log_out async error: {e}")
        assert True


def test_log_out_sync(basalam_client):
    """Test log_out_sync method (GET)."""
    try:
        result = basalam_client.chat.log_out_sync(
            token=TEST_BOT_TOKEN
        )
        print(f"log_out_sync result: {result}")
        assert result is not None
        assert hasattr(result, 'ok')
        assert isinstance(result.ok, bool)
    except Exception as e:
        print(f"log_out_sync error: {e}")
        assert True


@pytest.mark.asyncio
async def test_log_out_post_async(basalam_client):
    """Test log_out_post async method (POST)."""
    try:
        result = await basalam_client.chat.log_out_post(
            token=TEST_BOT_TOKEN
        )
        print(f"log_out_post async result: {result}")
        assert result is not None
        assert hasattr(result, 'ok')
        assert isinstance(result.ok, bool)
    except Exception as e:
        print(f"log_out_post async error: {e}")
        assert True


def test_log_out_post_sync(basalam_client):
    """Test log_out_post_sync method (POST)."""
    try:
        result = basalam_client.chat.log_out_post_sync(
            token=TEST_BOT_TOKEN
        )
        print(f"log_out_post_sync result: {result}")
        assert result is not None
        assert hasattr(result, 'ok')
        assert isinstance(result.ok, bool)
    except Exception as e:
        print(f"log_out_post_sync error: {e}")
        assert True


# -------------------------------------------------------------------------
# Bot endpoints tests - deleteWebhook
# -------------------------------------------------------------------------

@pytest.mark.asyncio
async def test_delete_webhook_async(basalam_client):
    """Test delete_webhook async method (GET)."""
    try:
        result = await basalam_client.chat.delete_webhook(
            token=TEST_BOT_TOKEN
        )
        print(f"delete_webhook async result: {result}")
        assert result is not None
        assert hasattr(result, 'ok')
        assert isinstance(result.ok, bool)
    except Exception as e:
        print(f"delete_webhook async error: {e}")
        assert True


def test_delete_webhook_sync(basalam_client):
    """Test delete_webhook_sync method (GET)."""
    try:
        result = basalam_client.chat.delete_webhook_sync(
            token=TEST_BOT_TOKEN
        )
        print(f"delete_webhook_sync result: {result}")
        assert result is not None
        assert hasattr(result, 'ok')
        assert isinstance(result.ok, bool)
    except Exception as e:
        print(f"delete_webhook_sync error: {e}")
        assert True


@pytest.mark.asyncio
async def test_delete_webhook_post_async(basalam_client):
    """Test delete_webhook_post async method (POST)."""
    try:
        result = await basalam_client.chat.delete_webhook_post(
            token=TEST_BOT_TOKEN
        )
        print(f"delete_webhook_post async result: {result}")
        assert result is not None
        assert hasattr(result, 'ok')
        assert isinstance(result.ok, bool)
    except Exception as e:
        print(f"delete_webhook_post async error: {e}")
        assert True


def test_delete_webhook_post_sync(basalam_client):
    """Test delete_webhook_post_sync method (POST)."""
    try:
        result = basalam_client.chat.delete_webhook_post_sync(
            token=TEST_BOT_TOKEN
        )
        print(f"delete_webhook_post_sync result: {result}")
        assert result is not None
        assert hasattr(result, 'ok')
        assert isinstance(result.ok, bool)
    except Exception as e:
        print(f"delete_webhook_post_sync error: {e}")
        assert True


@pytest.mark.asyncio
async def test_delete_webhook_delete_async(basalam_client):
    """Test delete_webhook_delete async method (DELETE)."""
    try:
        result = await basalam_client.chat.delete_webhook_delete(
            token=TEST_BOT_TOKEN
        )
        print(f"delete_webhook_delete async result: {result}")
        assert result is not None
        assert hasattr(result, 'ok')
        assert isinstance(result.ok, bool)
    except Exception as e:
        print(f"delete_webhook_delete async error: {e}")
        assert True


def test_delete_webhook_delete_sync(basalam_client):
    """Test delete_webhook_delete_sync method (DELETE)."""
    try:
        result = basalam_client.chat.delete_webhook_delete_sync(
            token=TEST_BOT_TOKEN
        )
        print(f"delete_webhook_delete_sync result: {result}")
        assert result is not None
        assert hasattr(result, 'ok')
        assert isinstance(result.ok, bool)
    except Exception as e:
        print(f"delete_webhook_delete_sync error: {e}")
        assert True


# -------------------------------------------------------------------------
# Bot endpoints tests - getMe
# -------------------------------------------------------------------------

@pytest.mark.asyncio
async def test_get_me_async(basalam_client):
    """Test get_me async method (GET)."""
    try:
        result = await basalam_client.chat.get_me(
            token=TEST_BOT_TOKEN
        )
        print(f"get_me async result: {result}")
        assert result is not None
        assert hasattr(result, 'ok')
        assert isinstance(result.ok, bool)
    except Exception as e:
        print(f"get_me async error: {e}")
        assert True


def test_get_me_sync(basalam_client):
    """Test get_me_sync method (GET)."""
    try:
        result = basalam_client.chat.get_me_sync(
            token=TEST_BOT_TOKEN
        )
        print(f"get_me_sync result: {result}")
        assert result is not None
        assert hasattr(result, 'ok')
        assert isinstance(result.ok, bool)
    except Exception as e:
        print(f"get_me_sync error: {e}")
        assert True


@pytest.mark.asyncio
async def test_get_me_post_async(basalam_client):
    """Test get_me_post async method (POST)."""
    try:
        result = await basalam_client.chat.get_me_post(
            token=TEST_BOT_TOKEN
        )
        print(f"get_me_post async result: {result}")
        assert result is not None
        assert hasattr(result, 'ok')
        assert isinstance(result.ok, bool)
    except Exception as e:
        print(f"get_me_post async error: {e}")
        assert True


def test_get_me_post_sync(basalam_client):
    """Test get_me_post_sync method (POST)."""
    try:
        result = basalam_client.chat.get_me_post_sync(
            token=TEST_BOT_TOKEN
        )
        print(f"get_me_post_sync result: {result}")
        assert result is not None
        assert hasattr(result, 'ok')
        assert isinstance(result.ok, bool)
    except Exception as e:
        print(f"get_me_post_sync error: {e}")
        assert True



"""
Base client for making requests to the Basalam API.
"""
import json
from typing import Any, Dict, List, Optional, Union, TypeVar, Type
from urllib.parse import urljoin

import httpx
from pydantic import BaseModel

from .auth import BaseAuth
from .config import BasalamConfig
from .errors import BasalamError, BasalamAPIError, BasalamAuthError

# Type variable for response models
T = TypeVar('T', bound=BaseModel)


class BaseClient:
    """
    Base client for making requests to the Basalam API.

    This class handles HTTP requests, authentication, and error handling.
    It serves as the foundation for all service-specific clients.
    """

    def __init__(
            self,
            auth: BaseAuth,
            config: Optional[BasalamConfig] = None,
            service: Optional[str] = None,
    ):
        """
        Initialize the base client.
        """
        self.auth = auth
        self.config = config or BasalamConfig()
        self.service = service

        # Set the base URL for this service
        if service:
            self.base_url = self.config.get_service_url(service)
        else:
            self.base_url = self.config.base_url

    @staticmethod
    def _handle_http_error(e: httpx.HTTPStatusError) -> None:
        """Handle HTTP errors and convert them to Basalam exceptions."""
        # Print the response data for debugging
        try:
            response_data = e.response.json()
            print(
                f"API Error Response ({e.response.status_code}): {json.dumps(response_data, ensure_ascii=False, indent=2)}")
        except (json.JSONDecodeError, ValueError):
            print(f"API Error Response ({e.response.status_code}): {e.response.text}")

        if e.response.status_code == 401:
            raise BasalamAuthError(f"Authentication failed: {e}", response=e.response)

        try:
            error_data = e.response.json()
            error_message = error_data.get("message", str(e))
            error_code = error_data.get("code", e.response.status_code)
        except (json.JSONDecodeError, ValueError):
            error_message = str(e)
            error_code = e.response.status_code

        raise BasalamAPIError(
            message=error_message,
            status_code=e.response.status_code,
            code=error_code,
            response=e.response,
        )

    @staticmethod
    def _unwrap_response(data: Union[Dict, List]) -> Union[Dict, List]:
        """
        Unwrap response data if it's wrapped in a single-key dictionary.

        This handles cases where the API returns data wrapped in a single key,
        like {"openapi_raw_data": [...]} or any future wrapper format.

        Args:
            data: The response data to unwrap

        Returns:
            The unwrapped data if it was wrapped, otherwise the original data
        """
        # If it's a dict with exactإإly one key and the value is a list,
        # return the list value (unwrap it)
        if isinstance(data, dict) and len(data) == 1:
            key = next(iter(data))
            value = data[key]
            # Only unwrap if the single value is a list (common for wrapped list responses)
            # This preserves single-key dict responses that are meant to be dicts
            if isinstance(value, list):
                return value

        return data

    @staticmethod
    def _parse_response_data(
            response: httpx.Response,
            response_model: Optional[Type[T]] = None
    ) -> Union[Dict[str, Any], List[Dict[str, Any]], T]:
        """Parse response data and validate with model if provided."""
        # Handle empty responses
        if not response.content:
            return {}

        try:
            data = response.json()
        except json.JSONDecodeError:
            raise BasalamError(f"Invalid JSON response: {response.text}")

        # Parse the response using the provided model
        if response_model:
            if isinstance(data, list):
                return [response_model.model_validate(item) for item in data]
            return response_model.model_validate(data)

        return data

    async def request(
            self,
            method: str,
            path: str,
            params: Optional[Dict[str, Any]] = None,
            data: Optional[Dict[str, Any]] = None,
            json_data: Optional[Dict[str, Any]] = None,
            files: Optional[Dict[str, Any]] = None,
            headers: Optional[Dict[str, str]] = None,
            response_model: Optional[Type[T]] = None,
            require_auth: bool = True,
    ) -> Union[Dict[str, Any], List[Dict[str, Any]], T]:
        """
        Make an async request to the API.
        """
        url = urljoin(self.base_url, path)

        # Build headers: start with config headers, add auth headers if needed, then custom headers
        request_headers = self.config.get_headers().copy()

        if require_auth:
            auth_headers = await self.auth.get_auth_headers()
            request_headers.update(auth_headers)

        if headers:
            request_headers.update(headers)

        async with httpx.AsyncClient(
                timeout=self.config.timeout,
                follow_redirects=True,
        ) as client:
            try:
                response = await client.request(
                    method=method,
                    url=url,
                    headers=request_headers,
                    params=params,
                    data=data,
                    json=json_data,
                    files=files,
                )
                response.raise_for_status()

            except httpx.HTTPStatusError as e:
                self._handle_http_error(e)

            except httpx.RequestError as e:
                raise BasalamError(f"Request failed: {e}")

            return self._parse_response_data(response, response_model)

    def request_sync(
            self,
            method: str,
            path: str,
            params: Optional[Dict[str, Any]] = None,
            data: Optional[Dict[str, Any]] = None,
            json_data: Optional[Dict[str, Any]] = None,
            files: Optional[Dict[str, Any]] = None,
            headers: Optional[Dict[str, str]] = None,
            response_model: Optional[Type[T]] = None,
            require_auth: bool = True,
    ) -> Union[Dict[str, Any], List[Dict[str, Any]], T]:
        """
        Make a synchronous request to the API.
        """
        url = urljoin(self.base_url, path)

        # Build headers: start with config headers, add auth headers if needed, then custom headers
        request_headers = self.config.get_headers().copy()

        if require_auth:
            auth_headers = self.auth.get_auth_headers_sync()
            request_headers.update(auth_headers)

        if headers:
            request_headers.update(headers)

        with httpx.Client(
                timeout=self.config.timeout,
                follow_redirects=True,
        ) as client:
            try:
                response = client.request(
                    method=method,
                    url=url,
                    headers=request_headers,
                    params=params,
                    data=data,
                    json=json_data,
                    files=files,
                )
                response.raise_for_status()

            except httpx.HTTPStatusError as e:
                self._handle_http_error(e)

            except httpx.RequestError as e:
                raise BasalamError(f"Request failed: {e}")

            return self._parse_response_data(response, response_model)

    # HTTP method helpers
    async def _get(
            self,
            path: str,
            params: Optional[Dict[str, Any]] = None,
            headers: Optional[Dict[str, str]] = None,
            response_model: Optional[Type[T]] = None,
            require_auth: bool = True,
    ) -> Union[Dict[str, Any], List[Dict[str, Any]], T]:
        """Make a GET request."""
        return await self.request("GET", path, params=params, headers=headers, response_model=response_model,
                                  require_auth=require_auth)

    def _get_sync(
            self,
            path: str,
            params: Optional[Dict[str, Any]] = None,
            headers: Optional[Dict[str, str]] = None,
            response_model: Optional[Type[T]] = None,
            require_auth: bool = True,
    ) -> Union[Dict[str, Any], List[Dict[str, Any]], T]:
        """Make a synchronous GET request."""
        return self.request_sync("GET", path, params=params, headers=headers, response_model=response_model,
                                 require_auth=require_auth)

    async def _post(
            self,
            path: str,
            data: Optional[Dict[str, Any]] = None,
            json_data: Optional[Dict[str, Any]] = None,
            files: Optional[Dict[str, Any]] = None,
            headers: Optional[Dict[str, str]] = None,
            response_model: Optional[Type[T]] = None,
            require_auth: bool = True,
    ) -> Union[Dict[str, Any], List[Dict[str, Any]], T]:
        """Make a POST request."""
        return await self.request("POST", path, data=data, json_data=json_data, files=files, headers=headers,
                                  response_model=response_model, require_auth=require_auth)

    def _post_sync(
            self,
            path: str,
            data: Optional[Dict[str, Any]] = None,
            json_data: Optional[Dict[str, Any]] = None,
            files: Optional[Dict[str, Any]] = None,
            headers: Optional[Dict[str, str]] = None,
            response_model: Optional[Type[T]] = None,
            require_auth: bool = True,
    ) -> Union[Dict[str, Any], List[Dict[str, Any]], T]:
        """Make a synchronous POST request."""
        return self.request_sync("POST", path, data=data, json_data=json_data, files=files, headers=headers,
                                 response_model=response_model, require_auth=require_auth)

    async def _put(
            self,
            path: str,
            data: Optional[Dict[str, Any]] = None,
            json_data: Optional[Dict[str, Any]] = None,
            headers: Optional[Dict[str, str]] = None,
            response_model: Optional[Type[T]] = None,
            require_auth: bool = True,
    ) -> Union[Dict[str, Any], List[Dict[str, Any]], T]:
        """Make a PUT request."""
        return await self.request("PUT", path, data=data, json_data=json_data, headers=headers,
                                  response_model=response_model, require_auth=require_auth)

    def _put_sync(
            self,
            path: str,
            data: Optional[Dict[str, Any]] = None,
            json_data: Optional[Dict[str, Any]] = None,
            headers: Optional[Dict[str, str]] = None,
            response_model: Optional[Type[T]] = None,
            require_auth: bool = True,
    ) -> Union[Dict[str, Any], List[Dict[str, Any]], T]:
        """Make a synchronous PUT request."""
        return self.request_sync("PUT", path, data=data, json_data=json_data, headers=headers,
                                 response_model=response_model, require_auth=require_auth)

    async def _patch(
            self,
            path: str,
            data: Optional[Dict[str, Any]] = None,
            json_data: Optional[Dict[str, Any]] = None,
            headers: Optional[Dict[str, str]] = None,
            response_model: Optional[Type[T]] = None,
            require_auth: bool = True,
    ) -> Union[Dict[str, Any], List[Dict[str, Any]], T]:
        """Make a PATCH request."""
        return await self.request("PATCH", path, data=data, json_data=json_data, headers=headers,
                                  response_model=response_model, require_auth=require_auth)

    def _patch_sync(
            self,
            path: str,
            data: Optional[Dict[str, Any]] = None,
            json_data: Optional[Dict[str, Any]] = None,
            headers: Optional[Dict[str, str]] = None,
            response_model: Optional[Type[T]] = None,
            require_auth: bool = True,
    ) -> Union[Dict[str, Any], List[Dict[str, Any]], T]:
        """Make a synchronous PATCH request."""
        return self.request_sync("PATCH", path, data=data, json_data=json_data, headers=headers,
                                 response_model=response_model, require_auth=require_auth)

    async def _delete(
            self,
            path: str,
            params: Optional[Dict[str, Any]] = None,
            data: Optional[Dict[str, Any]] = None,
            json_data: Optional[Dict[str, Any]] = None,
            headers: Optional[Dict[str, str]] = None,
            response_model: Optional[Type[T]] = None,
            require_auth: bool = True,
    ) -> Union[Dict[str, Any], List[Dict[str, Any]], T]:
        """Make a DELETE request."""
        return await self.request("DELETE", path, params=params, data=data, json_data=json_data, headers=headers,
                                  response_model=response_model, require_auth=require_auth)

    def _delete_sync(
            self,
            path: str,
            params: Optional[Dict[str, Any]] = None,
            data: Optional[Dict[str, Any]] = None,
            json_data: Optional[Dict[str, Any]] = None,
            headers: Optional[Dict[str, str]] = None,
            response_model: Optional[Type[T]] = None,
            require_auth: bool = True,
    ) -> Union[Dict[str, Any], List[Dict[str, Any]], T]:
        """Make a synchronous DELETE request."""
        return self.request_sync("DELETE", path, params=params, data=data, json_data=json_data, headers=headers,
                                 response_model=response_model, require_auth=require_auth)



"""
Tests for the Core service client async functions.
"""
import pytest

from basalam_sdk import BasalamClient, PersonalToken
from basalam_sdk.config import BasalamConfig, Environment
from basalam_sdk.core.models import (
    CreateVendorSchema,
    UpdateVendorSchema,
    VendorLegalRequestSchema,
    UpdateShippingMethodSchema,
    ShippingMethodUpdateItem,
    GetVendorProductsSchema,
    ProductStatusInputEnum,
    UpdateVendorStatusSchema,
    ChangeVendorMobileRequestSchema,
    ChangeVendorMobileConfirmSchema,
    ProductRequestSchema,
    BatchUpdateProductsRequest,
    UpdateProductRequestItem,
    BulkProductsUpdateRequestSchema,
    ProductFilterSchema,
    BulkActionItem,
    UpdateProductVariationSchema,
    CreateDiscountRequestSchema,
    DeleteDiscountRequestSchema,
    DiscountProductFilterSchema,
    ConfirmCurrentUserMobileConfirmSchema,
    ChangeUserMobileRequestSchema,
    ChangeUserMobileConfirmSchema,
    UserCardsSchema,
    UserCardsOtpSchema,
    UserVerifyBankInformationSchema,
    UpdateUserBankInformationSchema,
    UserVerificationSchema,
    UnitTypeInputEnum,
    ProductBulkFieldInputEnum,
    ProductBulkActionTypeEnum,
    ShelveSchema,
    UpdateShelveProductsSchema,
)

# Test IDs (you'll need valid IDs for testing)
TEST_USER_ID = 430
TEST_VENDOR_ID = 266
TEST_PRODUCT_ID = 24835037
TEST_CATEGORY_ID = 238
SHIPPING_METHOD_ID = 3198

# Additional test IDs
TEST_BANK_ACCOUNT_ID = 54321
TEST_SHELVE_ID = 531907


@pytest.fixture
def basalam_client():
    """Create a BasalamClient instance with real auth and config."""
    config = BasalamConfig(
        environment=Environment.PRODUCTION,
        timeout=30.0,
        user_agent="SDK-Test"
    )
    auth = PersonalToken(
        token=""
    )
    return BasalamClient(auth=auth, config=config)


# -------------------------------------------------------------------------
# Vendor Management endpoints tests
# -------------------------------------------------------------------------

@pytest.mark.asyncio
async def test_create_vendor_async(basalam_client):
    """Test create_vendor async method."""
    try:
        request = CreateVendorSchema(
            title="Test Vendor",
            identifier="test-vendor-unique",
            category_type=1,
            city=1,
            notice="Test vendor notice",
            summary="Test vendor summary",
            address="Test address",
            postal_code="1234567890",
            legal_data=VendorLegalRequestSchema(is_legal=False)
        )
        result = await basalam_client.core.create_vendor(TEST_USER_ID, request)
        print(f"create_vendor async result: {result}")
        assert hasattr(result, 'id')

    except Exception as e:
        print(f"create_vendor async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


@pytest.mark.asyncio
async def test_update_vendor_async(basalam_client):
    """Test update_vendor async method."""
    try:
        request = UpdateVendorSchema(
            title="Updated Test Vendor",
        )
        result = await basalam_client.core.update_vendor(TEST_VENDOR_ID, request)
        print(f"update_vendor async result: {result}")
        assert hasattr(result, 'id')

    except Exception as e:
        print(f"update_vendor async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


@pytest.mark.asyncio
async def test_get_vendor_async(basalam_client):
    """Test get_vendor async method."""
    try:
        # Test with minimal response
        result_minimal = await basalam_client.core.get_vendor(TEST_VENDOR_ID)
        print(f"get_vendor async (minimal) result: {result_minimal}")
        assert hasattr(result_minimal, 'id')

        # Test with full
        result_full = await basalam_client.core.get_vendor(
            TEST_VENDOR_ID,
            prefer="return=full"
        )
        print(f"get_vendor async (full) result: {result_full}")
        assert hasattr(result_full, 'id')

    except Exception as e:
        print(f"get_vendor async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


@pytest.mark.asyncio
async def test_get_default_shipping_methods_async(basalam_client):
    """Test get_default_shipping_methods async method."""
    try:
        result = await basalam_client.core.get_default_shipping_methods()
        print(f"get_default_shipping_methods async result: {result}")
        assert isinstance(result, list)

    except Exception as e:
        print(f"get_default_shipping_methods async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


@pytest.mark.asyncio
async def test_get_shipping_methods_async(basalam_client):
    """Test get_shipping_methods async method."""
    try:
        # Test without filters
        result = await basalam_client.core.get_shipping_methods()
        print(f"get_shipping_methods async (no filters) result: {result}")
        assert hasattr(result, 'data')

        # Test with filters
        result_filtered = await basalam_client.core.get_shipping_methods(
            vendor_ids=[TEST_VENDOR_ID],
            page=1,
            per_page=5
        )
        print(f"get_shipping_methods async (with filters) result: {result_filtered}")
        assert hasattr(result_filtered, 'data')

    except Exception as e:
        print(f"get_shipping_methods async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


@pytest.mark.asyncio
async def test_get_working_shipping_methods_async(basalam_client):
    """Test get_working_shipping_methods async method."""
    try:
        result = await basalam_client.core.get_working_shipping_methods(TEST_VENDOR_ID)
        print(f"get_working_shipping_methods async result: {result}")
        assert isinstance(result, list)

    except Exception as e:
        print(f"get_working_shipping_methods async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


@pytest.mark.asyncio
async def test_update_shipping_methods_async(basalam_client):
    """Test update_shipping_methods async method."""
    try:
        request = UpdateShippingMethodSchema(
            shipping_methods=[
                ShippingMethodUpdateItem(
                    method_id=SHIPPING_METHOD_ID,
                    is_customized=True,
                    base_cost=10000,
                    additional_cost=5000
                )
            ]
        )
        result = await basalam_client.core.update_shipping_methods(TEST_VENDOR_ID, request)
        print(f"update_shipping_methods async result: {result}")
        assert isinstance(result, list)

    except Exception as e:
        print(f"update_shipping_methods async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


@pytest.mark.asyncio
async def test_get_vendor_products_async(basalam_client):
    """Test get_vendor_products async method."""
    try:
        # Test without query params
        result = await basalam_client.core.get_vendor_products(TEST_VENDOR_ID)
        print(f"get_vendor_products async (no params) result: {result}")
        assert hasattr(result, 'data')

        # Test with query params
        query_params = GetVendorProductsSchema(
            page=1,
            per_page=5,
            statuses=[ProductStatusInputEnum.PUBLISHED],
            stock_gte=50,

        )
        result_filtered = await basalam_client.core.get_vendor_products(TEST_VENDOR_ID, query_params)
        print(f"get_vendor_products async (with params) result: {result_filtered}")
        assert hasattr(result_filtered, 'data')

    except Exception as e:
        print(f"get_vendor_products async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


@pytest.mark.asyncio
async def test_update_vendor_status_async(basalam_client):
    """Test update_vendor_status async method."""
    try:
        request = UpdateVendorStatusSchema(
            status=2986,
            description="Test status update"
        )
        result = await basalam_client.core.update_vendor_status(TEST_VENDOR_ID, request)
        print(f"update_vendor_status async result: {result}")
        assert hasattr(result, 'status')

    except Exception as e:
        print(f"update_vendor_status async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


@pytest.mark.asyncio
async def test_create_vendor_mobile_change_request_async(basalam_client):
    """Test create_vendor_mobile_change_request async method."""
    try:
        request = ChangeVendorMobileRequestSchema(
            mobile="09123456789"
        )
        result = await basalam_client.core.create_vendor_mobile_change_request(TEST_VENDOR_ID, request)
        print(f"create_vendor_mobile_change_request async result: {result}")
        assert hasattr(result, 'result')

    except Exception as e:
        print(f"create_vendor_mobile_change_request async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


@pytest.mark.asyncio
async def test_create_vendor_mobile_change_confirmation_async(basalam_client):
    """Test create_vendor_mobile_change_confirmation async method."""
    try:
        request = ChangeVendorMobileConfirmSchema(
            mobile="09123456789",
            verification_code=123456
        )
        result = await basalam_client.core.create_vendor_mobile_change_confirmation(TEST_VENDOR_ID, request)
        print(f"create_vendor_mobile_change_confirmation async result: {result}")
        assert hasattr(result, 'result')

    except Exception as e:
        print(f"create_vendor_mobile_change_confirmation async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


# -------------------------------------------------------------------------
# Product Management endpoints tests
# -------------------------------------------------------------------------

@pytest.mark.asyncio
async def test_create_product_async(basalam_client):
    """Test create_product async method."""
    try:
        with open("test1.png", "rb") as photo1, \
                open("test2.png", "rb") as photo2:
            request = ProductRequestSchema(
                name="Test sdk",
                description="blahahahahahahahahah ahhahahahah ahhahahah",
                category_id=TEST_CATEGORY_ID,
                primary_price=100000,
                weight=300,
                package_weight=500,
                stock=10,
                status=ProductStatusInputEnum.PUBLISHED,
                unit_quantity=10,
                unit_type=UnitTypeInputEnum.NUMERIC,
            )
            result = await basalam_client.core.create_product(TEST_VENDOR_ID, request, photo_files=[photo1, photo2])
            print(f"create_product async result: {result}")
            assert hasattr(result, 'id')

    except Exception as e:
        print(f"create_product async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


@pytest.mark.asyncio
async def test_update_bulk_products_async(basalam_client):
    """Test update_bulk_products async method."""
    try:
        request = BatchUpdateProductsRequest(
            data=[
                UpdateProductRequestItem(
                    id=TEST_PRODUCT_ID,
                    name="Updated Product Name",
                    primary_price=120000,
                    stock=15
                )
            ]
        )
        result = await basalam_client.core.update_bulk_products(TEST_VENDOR_ID, request)
        print(f"update_bulk_products async result: {result}")
        assert isinstance(result, list)

    except Exception as e:
        print(f"update_bulk_products async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


@pytest.mark.asyncio
async def test_update_product_async(basalam_client):
    """Test update_product async method."""
    try:
        request = ProductRequestSchema(
            name="Single Updated Product",
            brief="Updated brief",
            primary_price=150000,
            stock=20
        )
        result = await basalam_client.core.update_product(TEST_PRODUCT_ID, request)
        print(f"update_product async result: {result}")
        assert hasattr(result, 'id')

    except Exception as e:
        print(f"update_product async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


@pytest.mark.asyncio
async def test_get_product_async(basalam_client):
    """Test get_product async method."""
    try:
        # Test with minimal response
        result_minimal = await basalam_client.core.get_product(TEST_PRODUCT_ID)
        print(f"get_product async (minimal) result: {result_minimal}")
        assert hasattr(result_minimal, 'id')

        # Test with full
        result_full = await basalam_client.core.get_product(
            TEST_PRODUCT_ID,
            prefer="return=full"
        )
        print(f"get_product async (full) result: {result_full}")
        assert hasattr(result_full, 'id')

    except Exception as e:
        print(f"get_product async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


@pytest.mark.asyncio
async def test_get_products_async(basalam_client):
    """Test get_products async method."""
    try:
        # Test without query params
        result = await basalam_client.core.get_products()
        print(f"get_products async (no params) result: {result}")
        assert hasattr(result, 'data')

        # Test with query params (need to import GetProductsQuerySchema)
        from basalam_sdk.core.models import GetProductsQuerySchema
        query_params = GetProductsQuerySchema(
            page=1,
            per_page=5,
            sort="id:desc",
            vendor_ids=[TEST_VENDOR_ID]
        )
        result_filtered = await basalam_client.core.get_products(query_params)
        print(f"get_products async (with params) result: {result_filtered}")
        assert hasattr(result_filtered, 'data')

    except Exception as e:
        print(f"get_products async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


@pytest.mark.asyncio
async def test_create_products_bulk_action_request_async(basalam_client):
    """Test create_products_bulk_action_request async method."""
    try:
        request = BulkProductsUpdateRequestSchema(
            product_filter=ProductFilterSchema(
                product_id=[TEST_PRODUCT_ID]
            ),
            action=[
                BulkActionItem(
                    field=ProductBulkFieldInputEnum.STOCK,
                    action=ProductBulkActionTypeEnum.SET,
                    value=1
                )
            ]
        )
        result = await basalam_client.core.create_products_bulk_action_request(TEST_VENDOR_ID, request)
        print(f"create_products_bulk_action_request async result: {result}")
        assert hasattr(result, 'id')

    except Exception as e:
        print(f"create_products_bulk_action_request async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


@pytest.mark.asyncio
async def test_update_product_variation_async(basalam_client):
    """Test update_product_variation async method."""
    try:
        variation_id = 6639697  # Test variation ID
        request = UpdateProductVariationSchema(
            primary_price=80000,
            stock=25,
            sku="TEST-VAR-001"
        )
        result = await basalam_client.core.update_product_variation(
            TEST_PRODUCT_ID, variation_id, request
        )
        print(f"update_product_variation async result: {result}")
        assert hasattr(result, 'id')

    except Exception as e:
        print(f"update_product_variation async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


@pytest.mark.asyncio
async def test_get_products_bulk_action_requests_async(basalam_client):
    """Test get_products_bulk_action_requests async method."""
    try:
        result = await basalam_client.core.get_products_bulk_action_requests(
            TEST_VENDOR_ID,
            page=1,
            per_page=10
        )
        print(f"get_products_bulk_action_requests async result: {result}")
        assert hasattr(result, 'data')

    except Exception as e:
        print(f"get_products_bulk_action_requests async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


@pytest.mark.asyncio
async def test_get_products_bulk_action_requests_count_async(basalam_client):
    """Test get_products_bulk_action_requests_count async method."""
    try:
        result = await basalam_client.core.get_products_bulk_action_requests_count(TEST_VENDOR_ID)
        print(f"get_products_bulk_action_requests_count async result: {result}")
        assert hasattr(result, 'sum')

    except Exception as e:
        print(f"get_products_bulk_action_requests_count async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


@pytest.mark.asyncio
async def test_get_products_unsuccessful_bulk_action_requests_async(basalam_client):
    """Test get_products_unsuccessful_bulk_action_requests async method."""
    try:
        request_id = 561338  # Test request ID
        result = await basalam_client.core.get_products_unsuccessful_bulk_action_requests(
            request_id
        )
        print(f"get_products_unsuccessful_bulk_action_requests async result: {result}")
        assert hasattr(result, 'data')

    except Exception as e:
        print(f"get_products_unsuccessful_bulk_action_requests async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


@pytest.mark.asyncio
async def test_get_product_shelves_async(basalam_client):
    """Test get_product_shelves async method."""
    try:
        result = await basalam_client.core.get_product_shelves(TEST_PRODUCT_ID)
        print(f"get_product_shelves async result: {result}")
        assert isinstance(result, list)

    except Exception as e:
        print(f"get_product_shelves async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


@pytest.mark.asyncio
async def test_create_discount_async(basalam_client):
    """Test create_discount async method."""
    try:
        request = CreateDiscountRequestSchema(
            product_filter=DiscountProductFilterSchema(
                product_ids=[TEST_PRODUCT_ID],
            ),
            discount_percent=10,
            active_days=7
        )
        result = await basalam_client.core.create_discount(TEST_VENDOR_ID, request)
        print(f"create_discount async result: {result}")
        assert isinstance(result, dict)

    except Exception as e:
        print(f"create_discount async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


@pytest.mark.asyncio
async def test_delete_discount_async(basalam_client):
    """Test delete_discount async method."""
    try:
        request = DeleteDiscountRequestSchema(
            product_filter=DiscountProductFilterSchema(
                product_ids=[TEST_PRODUCT_ID]
            )
        )
        result = await basalam_client.core.delete_discount(TEST_VENDOR_ID, request)
        print(f"delete_discount async result: {result}")
        assert isinstance(result, dict)

    except Exception as e:
        print(f"delete_discount async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


# -------------------------------------------------------------------------
# User Management endpoints tests
# -------------------------------------------------------------------------

@pytest.mark.asyncio
async def test_get_current_user_async(basalam_client):
    """Test get_current_user async method."""
    try:
        result = await basalam_client.core.get_current_user()
        print(f"get_current_user async result: {result}")
        assert hasattr(result, 'id')

    except Exception as e:
        print(f"get_current_user async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


@pytest.mark.asyncio
async def test_create_user_mobile_confirmation_request_async(basalam_client):
    """Test create_user_mobile_confirmation_request async method."""
    try:
        result = await basalam_client.core.create_user_mobile_confirmation_request(TEST_USER_ID)
        print(f"create_user_mobile_confirmation_request async result: {result}")
        assert hasattr(result, 'result')

    except Exception as e:
        print(f"create_user_mobile_confirmation_request async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


@pytest.mark.asyncio
async def test_verify_user_mobile_confirmation_request_async(basalam_client):
    """Test verify_user_mobile_confirmation_request async method."""
    try:
        request = ConfirmCurrentUserMobileConfirmSchema(
            verification_code=123456
        )
        result = await basalam_client.core.verify_user_mobile_confirmation_request(TEST_USER_ID, request)
        print(f"verify_user_mobile_confirmation_request async result: {result}")
        assert hasattr(result, 'result')

    except Exception as e:
        print(f"verify_user_mobile_confirmation_request async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


@pytest.mark.asyncio
async def test_create_user_mobile_change_request_async(basalam_client):
    """Test create_user_mobile_change_request async method."""
    try:
        request = ChangeUserMobileRequestSchema(
            mobile="09120000000"
        )
        result = await basalam_client.core.create_user_mobile_change_request(TEST_USER_ID, request)
        print(f"create_user_mobile_change_request async result: {result}")
        assert hasattr(result, 'result')

    except Exception as e:
        print(f"create_user_mobile_change_request async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


@pytest.mark.asyncio
async def test_verify_user_mobile_change_request_async(basalam_client):
    """Test verify_user_mobile_change_request async method."""
    try:
        request = ChangeUserMobileConfirmSchema(
            mobile="09123456789",
            verification_code=123456
        )
        result = await basalam_client.core.verify_user_mobile_change_request(TEST_USER_ID, request)
        print(f"verify_user_mobile_change_request async result: {result}")
        assert hasattr(result, 'result')

    except Exception as e:
        print(f"verify_user_mobile_change_request async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


@pytest.mark.asyncio
async def test_get_user_bank_accounts_async(basalam_client):
    """Test get_user_bank_accounts async method."""
    try:
        # Test with minimal response
        result_minimal = await basalam_client.core.get_user_bank_accounts(TEST_USER_ID)
        print(f"get_user_bank_accounts async (minimal) result: {result_minimal}")
        assert isinstance(result_minimal, list)

    except Exception as e:
        print(f"get_user_bank_accounts async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


@pytest.mark.asyncio
async def test_create_user_bank_account_async(basalam_client):
    """Test create_user_bank_account async method."""
    try:
        request = UserCardsSchema(
            card_number="6063733231170311",
        )
        # Test with minimal response
        result_minimal = await basalam_client.core.create_user_bank_account(TEST_USER_ID, request)
        print(f"create_user_bank_account async (minimal) result: {result_minimal}")
        assert isinstance(result_minimal, dict)

    except Exception as e:
        print(f"create_user_bank_account async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


@pytest.mark.asyncio
async def test_verify_user_bank_account_otp_async(basalam_client):
    """Test verify_user_bank_account_otp async method."""
    try:
        request = UserCardsOtpSchema(
            card_number="5063731241170310",
            otp_code="123456"
        )
        result = await basalam_client.core.verify_user_bank_account_otp(TEST_USER_ID, request)
        print(f"verify_user_bank_account_otp async result: {result}")
        assert isinstance(result, dict)

    except Exception as e:
        print(f"verify_user_bank_account_otp async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


@pytest.mark.asyncio
async def test_verify_user_bank_account_async(basalam_client):
    """Test verify_user_bank_account async method."""
    try:
        request = UserVerifyBankInformationSchema(
            bank_information_id=TEST_BANK_ACCOUNT_ID,
            national_code="0123456789",
            birthday="1990-01-01"
        )
        result = await basalam_client.core.verify_user_bank_account(TEST_USER_ID, request)
        print(f"verify_user_bank_account async result: {result}")
        assert isinstance(result, dict)

    except Exception as e:
        print(f"verify_user_bank_account async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


@pytest.mark.asyncio
async def test_delete_user_bank_account_async(basalam_client):
    """Test delete_user_bank_account async method."""
    try:
        result = await basalam_client.core.delete_user_bank_account(TEST_USER_ID, TEST_BANK_ACCOUNT_ID)
        print(f"delete_user_bank_account async result: {result}")
        assert isinstance(result, list)

    except Exception as e:
        print(f"delete_user_bank_account async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


@pytest.mark.asyncio
async def test_update_user_bank_account_async(basalam_client):
    """Test update_user_bank_account async method."""
    try:
        request = UpdateUserBankInformationSchema(
            user_id=TEST_USER_ID,
            card_number="9876543210987654",
            sheba_number="IR123456789012345678901234",
            account_owner="Updated Test User",
            status=1,
            bank_name="Test Bank",
            sheba_status="active",
            bank_account_number="123456789012"
        )
        result = await basalam_client.core.update_user_bank_account(TEST_BANK_ACCOUNT_ID, request)
        print(f"update_user_bank_account async result: {result}")
        assert isinstance(result, dict)

    except Exception as e:
        print(f"update_user_bank_account async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


@pytest.mark.asyncio
async def test_update_user_verification_async(basalam_client):
    """Test update_user_verification async method."""
    try:
        request = UserVerificationSchema(
            national_code="0123456789",
            birthday="1990-01-01"
        )
        result = await basalam_client.core.update_user_verification(TEST_USER_ID, request)
        print(f"update_user_verification async result: {result}")
        assert hasattr(result, 'id')

    except Exception as e:
        print(f"update_user_verification async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


# -------------------------------------------------------------------------
# Category Management endpoints tests
# -------------------------------------------------------------------------

@pytest.mark.asyncio
async def test_get_category_attributes_async(basalam_client):
    """Test get_category_attributes async method."""
    try:
        # Test with minimal parameters
        result = await basalam_client.core.get_category_attributes(TEST_CATEGORY_ID)
        print(f"get_category_attributes async (minimal) result: {result}")
        assert hasattr(result, 'data')

        # Test with all parameters
        result_full = await basalam_client.core.get_category_attributes(
            TEST_CATEGORY_ID,
            product_id=TEST_PRODUCT_ID,
            vendor_id=TEST_VENDOR_ID,
            exclude_multi_selects=False
        )
        print(f"get_category_attributes async (full) result: {result_full}")
        assert hasattr(result_full, 'data')

    except Exception as e:
        print(f"get_category_attributes async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


@pytest.mark.asyncio
async def test_get_categories_async(basalam_client):
    """Test get_categories async method."""
    try:
        result = await basalam_client.core.get_categories()
        print(f"get_categories async result: {result}")
        assert hasattr(result, 'data')

    except Exception as e:
        print(f"get_categories async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


@pytest.mark.asyncio
async def test_get_category_async(basalam_client):
    """Test get_category async method."""
    try:
        result = await basalam_client.core.get_category(1066)
        print(f"get_category async result: {result}")
        assert hasattr(result, 'id')

    except Exception as e:
        print(f"get_category async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


# -------------------------------------------------------------------------
# Shelve Management endpoints tests
# -------------------------------------------------------------------------

@pytest.mark.asyncio
async def test_create_shelve_async(basalam_client):
    """Test create_shelve async method."""
    try:
        request = ShelveSchema(
            title="Test SDK Shelve",
            description="This is a test shelve for SDK testing"
        )
        result = await basalam_client.core.create_shelve(request)
        print(f"create_shelve async result: {result}")
        assert isinstance(result, dict)

    except Exception as e:
        print(f"create_shelve async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


@pytest.mark.asyncio
async def test_update_shelve_async(basalam_client):
    """Test update_shelve async method."""
    try:
        request = ShelveSchema(
            title="Test SDK Shelve-updated",
            description="This is a test shelve for SDK testing"
        )
        result = await basalam_client.core.update_shelve(TEST_SHELVE_ID, request)
        print(f"update_shelve async result: {result}")
        assert isinstance(result, dict)

    except Exception as e:
        print(f"update_shelve async error: {e}")
        assert True


@pytest.mark.asyncio
async def test_delete_shelve_async(basalam_client):
    """Test delete_shelve async method."""
    try:
        result = await basalam_client.core.delete_shelve(TEST_SHELVE_ID)
        print(f"delete_shelve async result: {result}")
        assert isinstance(result, dict)

    except Exception as e:
        print(f"delete_shelve async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


@pytest.mark.asyncio
async def test_get_shelve_products_async(basalam_client):
    """Test get_shelve_products async method."""
    try:
        # Test without title filter
        result = await basalam_client.core.get_shelve_products(TEST_SHELVE_ID)
        print(f"get_shelve_products async (no filter) result: {result}")
        assert isinstance(result, dict)

    except Exception as e:
        print(f"get_shelve_products async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


@pytest.mark.asyncio
async def test_get_shelve_products_async(basalam_client):
    """Test get_shelve_products async method."""
    try:
        # Test without title filter
        result = await basalam_client.core.get_shelve_products(TEST_SHELVE_ID)
        print(f"get_shelve_products async (no filter) result: {result}")
        assert isinstance(result, dict)

    except Exception as e:
        print(f"get_shelve_products async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


@pytest.mark.asyncio
async def test_update_shelve_products_async(basalam_client):
    """Test update_shelve_products async method."""
    try:
        # Test including products
        request = UpdateShelveProductsSchema(
            include_products=[TEST_PRODUCT_ID],
            exclude_products=[]
        )
        result = await basalam_client.core.update_shelve_products(TEST_SHELVE_ID, request)
        print(f"update_shelve_products async result: {result}")
        assert isinstance(result, dict)

    except Exception as e:
        print(f"update_shelve_products async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True


@pytest.mark.asyncio
async def test_delete_shelve_product_async(basalam_client):
    """Test delete_shelve_product async method."""
    try:
        result = await basalam_client.core.delete_shelve_product(TEST_SHELVE_ID, 24835037)
        print(f"delete_shelve_product async result: {result}")
        assert isinstance(result, dict)

    except Exception as e:
        print(f"delete_shelve_product async error: {e}")
        # Don't fail the test for API errors, just log them
        assert True



"""
Tests for the Search service client.
"""
import pytest

from basalam_sdk import BasalamClient
from basalam_sdk.auth import PersonalToken
from basalam_sdk.config import BasalamConfig, Environment
from basalam_sdk.search.models import ProductSearchModel, FiltersModel


@pytest.fixture
def basalam_client():
    """Create a BasalamClient instance with real auth and config."""
    config = BasalamConfig(
        environment=Environment.PRODUCTION,
        timeout=30.0,
        user_agent="SDK-Test"
    )
    auth = PersonalToken(
        token=""
    )
    return BasalamClient(auth=auth, config=config)


@pytest.mark.asyncio
async def test_search_products_async(basalam_client):
    """Test searching products asynchronously."""
    # Create search request
    search_request = ProductSearchModel(
        q="سیب",
        filters=FiltersModel(
            maxPrice=100000,
            minPrice=10000,
            minRating=4
        )
    )

    # Call the method
    result = await basalam_client.search_products(search_request)

    # Print the result
    print(f"Async search result: {result}")

    # Check response
    assert isinstance(result, dict)
    assert result is not None


def test_search_products_sync(basalam_client):
    """Test searching products synchronously."""
    # Create search request
    search_request = ProductSearchModel(
        q="laptop",
        rows=10,
        start=0,
        filters=FiltersModel(
            maxPrice=100000,
            minPrice=10000,
            minRating=4
        )
    )

    # Call the method
    result = basalam_client.search_products_sync(search_request)

    # Print the result
    print(f"Sync search result: {result}")

    # Check response
    assert isinstance(result, dict)
    assert result is not None
